@@ -35,7 +35,7 @@
 #include <stddef.h>
 
 #include <sofia-sip/su_configure.h>
-#include <tport_tls.h>
+#include "tport_tls.h"
 #include "sofia-sip/sofia_features.h"
 
 /** The name and version of software package providing Sofia-SIP-UA library.
