@@ -76,6 +76,7 @@ static char const __func__[] = "auth_mod";
 
 char const auth_internal_server_error[] = "Internal server error";
 
+static void auth_call_scheme_destructor(void *);
 static void auth_md5_hmac_key(auth_mod_t *am);
 
 HTABLE_PROTOS_WITH(auth_htable, aht, auth_passwd_t, usize_t, unsigned);
@@ -94,7 +95,7 @@ auth_mod_t *auth_mod_alloc(auth_scheme_t *scheme,
 
   if ((am = su_home_new(scheme->asch_size))) {
     am->am_scheme = scheme;
-    am->am_refcount = 1;
+    su_home_destructor(am->am_home, auth_call_scheme_destructor);
   }
 
   return am;
@@ -132,6 +133,7 @@ int auth_init_default(auth_mod_t *am,
   char const *realm = NULL, *opaque = NULL, *db = NULL, *allows = NULL;
   char const *qop = NULL, *algorithm = NULL;
   unsigned expires = 60 * 60, next_expires = 5 * 60;
+  unsigned max_ncount = 0;
   unsigned blacklist = 5;
   int forbidden = 0;
   int anonymous = 0;
@@ -151,6 +153,7 @@ int auth_init_default(auth_mod_t *am,
 	  AUTHTAG_ALGORITHM_REF(algorithm),
 	  AUTHTAG_EXPIRES_REF(expires),
 	  AUTHTAG_NEXT_EXPIRES_REF(next_expires),
+	  AUTHTAG_MAX_NCOUNT_REF(max_ncount),
 	  AUTHTAG_BLACKLIST_REF(blacklist),
 	  AUTHTAG_FORBIDDEN_REF(forbidden),
 	  AUTHTAG_ANONYMOUS_REF(anonymous),
@@ -171,6 +174,7 @@ int auth_init_default(auth_mod_t *am,
     msg_commalist_d(am->am_home, &s, &am->am_allow, NULL);
   am->am_expires = expires;
   am->am_next_exp = next_expires;
+  am->am_max_ncount = max_ncount;
   am->am_blacklist = blacklist;
   am->am_forbidden = forbidden;
   am->am_anonymous = anonymous;
@@ -246,10 +250,14 @@ int auth_init_default(auth_mod_t *am,
 /** Destroy (a reference to) an authentication module. */
 void auth_mod_destroy(auth_mod_t *am)
 {
-  if (am && am->am_refcount != 0 && --am->am_refcount == 0) {
-    am->am_scheme->asch_destroy(am);
-    su_home_zap(am->am_home);
-  }
+  su_home_unref(am->am_home);
+}
+
+/** Call scheme-specific destructor function. */
+static void auth_call_scheme_destructor(void *arg)
+{
+  auth_mod_t *am = arg;
+  am->am_scheme->asch_destroy(am);
 }
 
 /** Do-nothing destroy function.
@@ -264,18 +272,13 @@ void auth_destroy_default(auth_mod_t *am)
 /** Create a new reference to authentication module. */
 auth_mod_t *auth_mod_ref(auth_mod_t *am)
 {
-  if (!am || am->am_refcount == 0)
-    return NULL;
-
-  am->am_refcount++;
-
-  return am;
+  return (auth_mod_t *)su_home_ref(am->am_home);
 }
 
 /** Destroy a reference to an authentication module. */
 void auth_mod_unref(auth_mod_t *am)
 {
-  auth_mod_destroy(am);
+  su_home_unref(am->am_home);
 }
 
 /** Get authenticatin module name. @NEW_1_12_4. */
@@ -534,11 +537,10 @@ void auth_method_basic(auth_mod_t *am,
     if (n < 0 || n >= INT_MAX)
       continue;
     if (n >= upsize) {
-      upsize = n + 1;
-      userpass = realloc(userpass == buffer ? NULL : userpass, upsize);
-      if (userpass == NULL)
-	continue;
-      base64_d(userpass, upsize - 1, au->au_params[0]);
+      void *b = realloc(userpass == buffer ? NULL : userpass, upsize = n + 1);
+      if (b == NULL)
+	break;
+      base64_d(userpass = b, upsize - 1, au->au_params[0]);
     }
     userpass[n] = 0;
     if (!(pass = strchr(userpass, ':')))
@@ -608,7 +610,7 @@ struct nonce {
   uint8_t    digest[6];
 };
 
-#define AUTH_DIGEST_NONCE_LEN (BASE64_SIZE(sizeof (struct nonce)) + 1)
+#define AUTH_DIGEST_NONCE_LEN (BASE64_MINSIZE(sizeof (struct nonce)) + 1)
 
 /** Authenticate a request with @b Digest authentication scheme.
  *
@@ -831,7 +833,7 @@ void auth_info_digest(auth_mod_t *am,
 /* ====================================================================== */
 /* Password database */
 
-static inline void
+su_inline void
 auth_htable_append_local(auth_htable_t *pr, auth_passwd_t *apw);
 
 /** Get an passwd entry for user. */
@@ -950,7 +952,8 @@ int auth_readdb_if_needed(auth_mod_t *am)
 #include <sys/file.h>
 #endif
 
-#define auth_apw_local auth_readdb_internal
+/* This is just a magic value */
+#define auth_apw_local ((void *)(intptr_t)auth_readdb_internal)
 
 /** Read authentication database */
 static
@@ -1029,7 +1032,8 @@ int auth_readdb_internal(auth_mod_t *am, int always)
     N = i, i = 0;
 
     if (N > 0) {
-      if (auth_htable_resize(am->am_home, am->am_users, N) < 0 ||
+      size_t size = (N * 5 + 3) / 4;
+      if (auth_htable_resize(am->am_home, am->am_users, size) < 0 ||
 	  !(fresh = su_zalloc(am->am_home, sizeof(*fresh) * N))) {
 	su_free(am->am_home, buffer);
 	return -1;
@@ -1074,13 +1078,15 @@ int auth_readdb_internal(auth_mod_t *am, int always)
       if (!*pass || !*user)
 	continue;
 
-      realm = ""; ident = "";
-
-      if ((realm = strchr(pass, ':'))) {
+      if ((realm = strchr(pass, ':')))
 	*realm++ = '\0';
-	if ((ident = strchr(realm, ':')))
-	  *ident++ = '\0';
-      }
+      else
+	realm = "";
+
+      if ((ident = strchr(realm, ':')))
+	*ident++ = '\0';
+      else
+	ident = "";
 
       apw = fresh + i++;
 
@@ -1130,8 +1136,8 @@ int auth_readdb_internal(auth_mod_t *am, int always)
   return -1;
 }
 
-/** Append to hash, remove existing user */
-static inline void
+/** Append to hash, remove existing local user */
+su_inline void
 auth_htable_append_local(auth_htable_t *aht, auth_passwd_t *apw)
 {
   auth_passwd_t **slot;
@@ -1436,6 +1442,16 @@ int auth_validate_digest_nonce(auth_mod_t *am,
     as->as_stale = 1;
   }
 
+  if (am->am_max_ncount && ar->ar_nc) {
+    unsigned long nc = strtoul(ar->ar_nc, NULL, 10);
+
+    if (nc == 0 || nc > am->am_max_ncount) {
+      SU_DEBUG_5(("auth_method_digest: nonce used %s times, max %u\n",
+		  ar->ar_nc, am->am_max_ncount));
+      as->as_stale = 1;
+    }
+  }
+
   /* We should also check cnonce, nc... */
 
   return 0;
