@@ -191,9 +191,9 @@ issize_t msg_any_list_d(su_home_t *home,
 			issize_t (*scanner)(char *s), 
 			int sep)
 {
-  char const *auto_list[MSG_N_PARAMS];
-  char const **list = auto_list, **re_list;
-  int N = MSG_N_PARAMS, n = 0;
+  char const *stack[MSG_N_PARAMS];
+  char const **list = stack, **re_list;
+  size_t N = MSG_N_PARAMS, n = 0;
   issize_t tlen;
   char *s = *ss;
   char const **start;
@@ -221,7 +221,7 @@ issize_t msg_any_list_d(su_home_t *home,
     if (tlen > 0) {
       if (n + 1 == N) {		/* Reallocate list? */
 	N = MSG_PARAMS_NUM(N + 1);
-	if (list == auto_list || list == *append_list) {
+	if (list == stack || list == *append_list) {
 	  re_list = su_alloc(home, N * sizeof(*list));
 	  if (re_list)
 	    memcpy(re_list, list, n * sizeof(*list));
@@ -247,23 +247,25 @@ issize_t msg_any_list_d(su_home_t *home,
 
   *ss = s;
 
-  if (n > 0 && list == auto_list) {
+  if (n == 0) {
+    *append_list = NULL;
+    return 0;
+  }
+
+  if (list == stack) {
     size_t size = sizeof(*list) * MSG_PARAMS_NUM(n + 1);
     list = su_alloc(home, size);
     if (!list) return -1;
-    memcpy((void *)list, auto_list, n * sizeof(*list));
+    memcpy((void *)list, stack, n * sizeof(*list));
   }
 
   list[n] = NULL;
-  if (n == 0)
-    list = NULL;
-
   *append_list = list;
   return 0;
 
  error:
   *start = NULL;
-  if (list != auto_list && list != *append_list)
+  if (list != stack && list != *append_list)
     su_free(home, list);
   return -1;
 }
@@ -362,7 +364,8 @@ issize_t msg_avlist_d(su_home_t *home,
     for (n = 0; params[n]; n++)
       ;
     N = MSG_PARAMS_NUM(n + 1);
-  } else {
+  }
+  else {
     params = stack;
     N = MSG_PARAMS_NUM(1);
   }
@@ -1055,22 +1058,26 @@ int msg_header_remove_param(msg_common_t *h, char const *name)
  */
 int msg_header_update_params(msg_common_t *h, int clear)
 {
+  msg_hclass_t *hc;
+  unsigned char offset;
+  msg_update_f *update;
   int retval;
   msg_param_t const *params;
   size_t n;
   char const *p, *v;
-
+  
   if (h == NULL)
     return errno = EFAULT, -1;
 
-  if (h->h_class->hc_params == 0 ||
-      h->h_class->hc_update == NULL)
+  hc = h->h_class; offset = hc->hc_params; update = hc->hc_update;
+
+  if (offset == 0 || update == NULL)
     return 0;
 
   if (clear)
-    h->h_class->hc_update(h, NULL, 0, NULL);
+    update(h, NULL, 0, NULL);
 
-  params = *(msg_param_t **)((char *)h + h->h_class->hc_params);
+  params = *(msg_param_t **)((char *)h + offset);
   if (params == NULL)
     return 0;
 
@@ -1079,7 +1086,7 @@ int msg_header_update_params(msg_common_t *h, int clear)
   for (p = *params; p; p = *++params) {
     n = strcspn(p, "=");
     v = p + n + (p[n] == '=');
-    if (h->h_class->hc_update(h, p, n, v) < 0)
+    if (update(h, p, n, v) < 0)
       retval = -1;
   }
 
@@ -1503,6 +1510,122 @@ issize_t msg_params_join(su_home_t *home,
   return 0;
 }
 
+/**Join header item lists.
+ *
+ * Join items from a source header to the destination header. The item are
+ * compared with the existing ones. If a match is found, it is not added to
+ * the list. If @a duplicate is true, the entries are duplicated while they
+ * are added to the list.
+ *
+ * @param home       memory home
+ * @param dst        destination header
+ * @param src        source header
+ * @param duplicate  if true, allocate and copy items that are added
+ *
+ * @return
+ * @retval >= 0 when successful
+ * @retval -1 upon an error
+ *
+ * @NEW_1_12_5.
+ */
+int msg_header_join_items(su_home_t *home,
+			  msg_common_t *dst,
+			  msg_common_t const *src,
+			  int duplicate)
+{
+  size_t N, m, M, i, n_before, n_after, total;
+  char *dup = NULL;
+  msg_param_t *d, **dd, *s;
+  msg_param_t t, *temp, temp0[16];
+  size_t *len, len0[(sizeof temp0)/(sizeof temp0[0])];
+  msg_update_f *update = NULL;
+
+  if (dst == NULL || dst->h_class->hc_params == 0 ||
+      src == NULL || src->h_class->hc_params == 0)
+    return -1;
+
+  s = *(msg_param_t **)((char *)src + src->h_class->hc_params);
+  if (s == NULL)
+    return 0;
+
+  for (M = 0; s[M]; M++);
+
+  if (M == 0)
+    return 0;
+
+  if (M <= (sizeof temp0) / (sizeof temp0[0])) {
+    temp = temp0, len = len0;
+  }
+  else {
+    temp = malloc(M * (sizeof *temp) + M * (sizeof *len));
+    if (!temp) return -1;
+    len = (void *)(temp + M);
+  }
+
+  dd = (msg_param_t **)((char *)dst + dst->h_class->hc_params);
+  d = *dd;
+
+  for (N = 0; d && d[N]; N++);
+
+  for (m = 0, M = 0, total = 0; s[m]; m++) {
+    t = s[m];
+    for (i = 0; i < N; i++)
+      if (strcmp(t, d[i]) == 0)
+	break;
+    if (i < N)
+      continue;
+
+    for (i = 0; i < M; i++)
+      if (strcmp(t, temp[i]) == 0)
+	break;
+    if (i < M)
+      continue;
+
+    temp[M] = t;
+    len[M] = strlen(t);
+    total += len[M++] + 1;
+  }
+
+  if (M == 0)
+    goto success;
+
+  dup = su_alloc(home, total); if (!dup) goto error;
+
+  n_before = MSG_PARAMS_NUM(N + 1);
+  n_after = MSG_PARAMS_NUM(N + M + 1);
+
+  if (d == NULL || n_before != n_after) {
+    d = su_alloc(home, n_after * sizeof(*d)); if (!d) goto error;
+    if (N)
+      memcpy(d, *dd, N * sizeof(*d));
+    *dd = d;
+  }
+
+  update = dst->h_class->hc_update;
+
+  for (m = 0; m < M; m++) {
+    d[N++] = memcpy(dup, temp[m], len[m] + 1);
+    
+    if (update)
+      update(dst, dup, len[m], dup + len[m]);
+
+    dup += len[m] + 1;
+  }  
+
+  d[N] = NULL;
+
+ success:
+  if (temp != temp0)
+    free(temp);
+  return 0;
+
+ error:
+  if (temp != temp0)
+    free(temp);
+  su_free(home, dup);
+  return -1;
+}
+
 /**Compare parameter lists.
  * 
  * Compares parameter lists.
@@ -1854,6 +1977,8 @@ msg_t *msg_make(msg_mclass_t const *mc, int flags,
     return NULL;
 
   msg = msg_create(mc, flags);
+  if (msg == NULL)
+    return NULL;
 
   su_home_preload(msg_home(msg), 1, len + 1024);
 
