@@ -100,6 +100,7 @@ typedef struct msg_numeric_s  	    msg_numeric_t;
 typedef struct msg_generic_s  	    msg_generic_t;
 typedef struct msg_list_s     	    msg_list_t;
 typedef struct msg_auth_s     	    msg_auth_t;
+typedef struct msg_auth_info_s      msg_auth_info_t;
 
 #define MSG_HEADER_N 16377
 
@@ -133,7 +134,8 @@ struct msg_pub_s {
 
 /** Numeric header. 
  *
- * A numeric header has a 32-bit integer as its value.
+ * A numeric header has value range of a 32-bit, 0..4294967295. The @a
+ * x_value field is unsigned long, however.
  */
 struct msg_numeric_s {
   msg_common_t   x_common[1];	    /**< Common fragment info */
@@ -174,6 +176,17 @@ struct msg_auth_s {
   msg_param_t const *au_params;	    /**< Comma-separated parameters */
 };
 
+/**Authentication-Info header
+ *
+ * An Authentication-Info header has comma-separated list of parameters as its value.
+ */
+struct msg_auth_info_s
+{
+  msg_common_t        ai_common[1]; /**< Common fragment info */
+  msg_error_t        *ai_next;	    /**< Dummy link to next */
+  msg_param_t const  *ai_params;    /**< List of ainfo */
+};
+
 /** Unknown header. */
 struct msg_unknown_s {
   msg_common_t    un_common[1];  /**< Common fragment info */
