@@ -43,8 +43,6 @@
 #include <sofia-sip/su_debug.h>
 
 #define SU_ROOT_MAGIC_T   struct nua_s
-#define SU_MSG_ARG_T      struct event_s
-#define NUA_SAVED_EVENT_T su_msg_t *
 
 #include <sofia-sip/sip_status.h>
 #include <sofia-sip/sip_header.h>
@@ -192,16 +190,9 @@ void nua_shutdown(nua_t *nua)
 
   if (nua)
     nua->nua_shutdown_started = 1;
-  nua_signal(nua, NULL, NULL, 1, nua_r_shutdown, 0, NULL, TAG_END());
+  nua_signal(nua, NULL, NULL, nua_r_shutdown, 0, NULL, TAG_END());
 }
 
-/** @internal Linked stack frames from nua event callback */
-struct nua_event_frame_s {
-  nua_event_frame_t *nf_next;
-  nua_t *nf_nua;
-  nua_saved_event_t nf_saved[1];
-};
-
 /** Destroy the @nua stack.
  *
  * Before calling nua_destroy() the application
@@ -227,8 +218,6 @@ void nua_destroy(nua_t *nua)
   enter;
 
   if (nua) {
-    nua_event_frame_t *nf;
-
     if (!nua->nua_shutdown_final) {
       SU_DEBUG_0(("nua_destroy(%p): FATAL: nua_shutdown not completed\n",
 		  (void *)nua));
@@ -238,11 +227,6 @@ void nua_destroy(nua_t *nua)
 
     nua->nua_callback = NULL;
 
-    for (nf = nua->nua_current; nf; nf = nf->nf_next) {
-      nf->nf_nua = NULL;
-    }
-    nua->nua_current = NULL;
-
     su_task_deinit(nua->nua_server);
     su_task_deinit(nua->nua_client);
 
@@ -590,7 +574,7 @@ void nua_set_params(nua_t *nua, tag_type_t tag, tag_value_t value, ...)
 
   enter;
 
-  nua_signal(nua, NULL, NULL, 0, nua_r_set_params, 0, NULL, ta_tags(ta));
+  nua_signal(nua, NULL, NULL, nua_r_set_params, 0, NULL, ta_tags(ta));
 
   ta_end(ta);
 }
@@ -603,7 +587,7 @@ void nua_get_params(nua_t *nua, tag_type_t tag, tag_value_t value, ...)
 
   enter;
 
-  nua_signal(nua, NULL, NULL, 0, nua_r_get_params, 0, NULL, ta_tags(ta));
+  nua_signal(nua, NULL, NULL, nua_r_get_params, 0, NULL, ta_tags(ta));
 
   ta_end(ta);
 }
@@ -613,7 +597,7 @@ void nua_get_params(nua_t *nua, tag_type_t tag, tag_value_t value, ...)
   if (NH_IS_VALID((nh))) { \
     ta_list ta; \
     ta_start(ta, tag, value); \
-    nua_signal((nh)->nh_nua, nh, NULL, 0, event, 0, NULL, ta_tags(ta));	\
+    nua_signal((nh)->nh_nua, nh, NULL, event, 0, NULL, ta_tags(ta));	\
     ta_end(ta); \
   } \
   else { \
@@ -897,7 +881,7 @@ void nua_respond(nua_handle_t *nh,
   if (NH_IS_VALID(nh)) {
     ta_list ta;
     ta_start(ta, tag, value);
-    nua_signal(nh->nh_nua, nh, NULL, 0, nua_r_respond,
+    nua_signal(nh->nh_nua, nh, NULL, nua_r_respond,
 	       status, phrase, ta_tags(ta));
     ta_end(ta);
   }
@@ -938,201 +922,7 @@ void nua_handle_destroy(nua_handle_t *nh)
 
   if (NH_IS_VALID(nh) && !NH_IS_DEFAULT(nh)) {
     nh->nh_valid = NULL;	/* Events are no more delivered to appl. */
-    nua_signal(nh->nh_nua, nh, NULL, 1, nua_r_destroy, 0, NULL, TAG_END());
-  }
-}
-
-/*# Send a request to the protocol thread */
-void nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg, int always,
-		nua_event_t event,
-		int status, char const *phrase,
-		tag_type_t tag, tag_value_t value, ...)
-{
-  su_msg_r sumsg = SU_MSG_R_INIT;
-  size_t len, xtra, e_len, l_len = 0, l_xtra = 0;
-  ta_list ta;
-
-  if (nua == NULL || (nua->nua_shutdown_started && event != nua_r_shutdown))
-    return;
-
-  ta_start(ta, tag, value);
-
-  e_len = offsetof(event_t, e_tags);
-  len = tl_len(ta_args(ta));
-  xtra = tl_xtra(ta_args(ta), len);
-
-  if (su_msg_create(sumsg, nua->nua_server, su_task_null,
-		    nua_stack_signal,
-		    e_len + len + l_len + xtra + l_xtra) == 0) {
-    event_t *e = su_msg_data(sumsg);
-    tagi_t *t = e->e_tags;
-    void *b = (char *)t + len + l_len;
-
-    tagi_t *tend = (tagi_t *)b;
-    char *bend = (char *)b + xtra + l_xtra;
-
-    t = tl_dup(t, ta_args(ta), &b);
-
-    assert(tend == t); (void)tend; assert(b == bend); (void)bend;
-
-    e->e_always = always;
-    e->e_event = event;
-    e->e_nh = event == nua_r_destroy ? nh : nua_handle_ref(nh);
-    e->e_status = status;
-    e->e_phrase = phrase;
-
-    SU_DEBUG_7(("nua(%p): signal %s\n", (void *)nh,
-		nua_event_name(event) + 4));
-
-    if (su_msg_send(sumsg) != 0 && event != nua_r_destroy)
-      nua_handle_unref(nh);
-  }
-  else {
-    /* XXX  - we should return error code to application but we just abort() */
-    assert(ENOMEM == 0);
-  }
-
-  ta_end(ta);
-}
-
-/*# Receive event from protocol machine and hand it over to application */
-void nua_event(nua_t *root_magic, su_msg_r sumsg, event_t *e)
-{
-  nua_t *nua;
-  nua_handle_t *nh = e->e_nh;
-  enter;
-
-  e->e_nh = NULL;
-
-  if (nh) {
-    if (!nh->nh_ref_by_user && nh->nh_valid) {
-      nh->nh_ref_by_user = 1;
-      nua_handle_ref(nh);
-    }
-  }
-
-  if (!nh || !nh->nh_valid) {	/* Handle has been destroyed */
-    if (nua_log->log_level >= 7) {
-      char const *name = nua_event_name(e->e_event) + 4;
-      SU_DEBUG_7(("nua(%p): event %s dropped\n", (void *)nh, name));
-    }
-    if (nh && !NH_IS_DEFAULT(nh) && nua_handle_unref(nh)) {
-#if HAVE_NUA_HANDLE_DEBUG
-      SU_DEBUG_0(("nua(%p): freed by application\n", (void *)nh));
-#else
-      SU_DEBUG_9(("nua(%p): freed by application\n", (void *)nh));
-#endif
-    }
-    if (e->e_msg)
-      msg_destroy(e->e_msg), e->e_msg = NULL;
-    return;
-  }
-
-  nua = nh->nh_nua; assert(nua);
-
-  if (NH_IS_DEFAULT(nh))
-    nh = NULL;
-
-  if (e->e_event == nua_r_shutdown && e->e_status >= 200)
-    nua->nua_shutdown_final = 1;
-
-  if (nua->nua_callback) {
-    nua_event_frame_t frame[1];
-    
-    su_msg_remove_refs(sumsg);    /* Remove references to tasks */
-    su_msg_save(frame->nf_saved, sumsg);
-    frame->nf_nua = nua;
-    frame->nf_next = nua->nua_current, nua->nua_current = frame;
-
-    nua->nua_callback(e->e_event, e->e_status, e->e_phrase,
-		      nua, nua->nua_magic,
-		      nh, nh ? nh->nh_magic : NULL,
-		      e->e_msg ? sip_object(e->e_msg) : NULL,
-		      e->e_tags);
-
-    su_msg_destroy(frame->nf_saved);
-
-    if (frame->nf_nua == NULL)
-      return;
-    nua->nua_current = frame->nf_next;
-  }
-
-  if (nh && nua_handle_unref(nh)) {
-#if HAVE_NUA_HANDLE_DEBUG
-    SU_DEBUG_0(("nua(%p): freed by application\n", (void *)nh));
-#else
-    SU_DEBUG_9(("nua(%p): freed by application\n", (void *)nh));
-#endif
-  }
-}
-
-/** Get current request message. @NEW_1_12_4.
- *
- * @note A response message is returned when processing response message.
- *
- * @sa #nua_event_e, nua_respond(), NUTAG_WITH_CURRENT()
- */
-msg_t *nua_current_request(nua_t const *nua)
-{
-  if (nua && nua->nua_current && su_msg_is_non_null(nua->nua_current->nf_saved))
-    return su_msg_data(nua->nua_current->nf_saved)->e_msg;
-  return NULL;
-}
-
-/** Get request message from saved nua event. @NEW_1_12_4. 
- *
- * @sa nua_save_event(), nua_respond(), NUTAG_WITH_SAVED(), 
- */
-msg_t *nua_saved_event_request(nua_saved_event_t const *saved)
-{
-  return saved && saved[0] ? su_msg_data(saved)->e_msg : NULL;
-}
-
-/** Save nua event and its arguments.
- *
- * @sa #nua_event_e, nua_event_data() nua_saved_event_request(), nua_destroy_event()
- */
-int nua_save_event(nua_t *nua, nua_saved_event_t return_saved[1])
-{
-  if (return_saved) {
-    if (nua && nua->nua_current) {
-      su_msg_save(return_saved, nua->nua_current->nf_saved);
-      return su_msg_is_non_null(return_saved);
-    }
-    else
-      *return_saved = NULL;
-  }
-
-  return 0;
-}
-
-/** Get event data.
- *
- * @sa #nua_event_e, nua_event_save(), nua_saved_event_request(), nua_destroy_event().
- */
-nua_event_data_t const *nua_event_data(nua_saved_event_t const saved[1])
-{
-  return saved ? su_msg_data(saved) : NULL;
-}
-
-/** Destroy saved event.
- *
- * @sa #nua_event_e, nua_event_save(), nua_event_data(), nua_saved_event_request().
- */
-void nua_destroy_event(nua_saved_event_t saved[1])
-{
-  if (su_msg_is_non_null(saved)) {
-    event_t *e = su_msg_data(saved);
-    nua_handle_t *nh = e->e_nh;
-
-    if (e->e_msg)
-      msg_destroy(e->e_msg), e->e_msg = NULL;
-
-    if (nh && !NH_IS_DEFAULT(nh) && nua_handle_unref(nh)) {
-      SU_DEBUG_9(("nua(%p): freed by application\n", (void *)nh));
-    }
-
-    su_msg_destroy(saved);
+    nua_signal(nh->nh_nua, nh, NULL, nua_r_destroy, 0, NULL, TAG_END());
   }
 }
 
@@ -1215,6 +1005,21 @@ static int nua_stack_handle_by_replaces_call(void *arg)
   return 0;
 }
 
+struct nua_stack_handle_by_call_id_args {
+  nua_handle_t *retval;
+  nua_t *nua;
+  const char *call_id;
+};
+
+static int nua_stack_handle_by_call_id_call(void *arg)
+{
+  struct nua_stack_handle_by_call_id_args *a = arg;
+
+  a->retval = nua_stack_handle_by_call_id(a->nua, a->call_id);
+
+  return 0;
+}
+
 /** Obtain a new reference to an existing handle based on @Replaces header.
  *
  * @since New in @VERSION_1_12_4.
@@ -1231,7 +1036,7 @@ nua_handle_t *nua_handle_by_replaces(nua_t *nua, sip_replaces_t const *r)
   if (nua) {
 #if HAVE_OPEN_C
     struct nua_stack_handle_by_replaces_args a;
-	a.retval = NULL;
+    a.retval = NULL;
     a.nua = nua;
     a.r = r;
 #else
@@ -1249,3 +1054,38 @@ nua_handle_t *nua_handle_by_replaces(nua_t *nua, sip_replaces_t const *r)
   }
   return NULL;
 }
+
+/** Obtain a new reference to an existing handle based on @CallID.
+ *
+ * @since New in @VERSION_1_12_9.
+ *
+ * @note
+ * You should release the reference with nua_handle_unref() when you are
+ * done with the handle.
+ *
+ * @sa nua_handle_make_replaces(), @Replaces, @RFC3891, nua_refer(),
+ * #nua_i_refer, @ReferTo, nta_leg_by_replaces()
+ */
+nua_handle_t *nua_handle_by_call_id(nua_t *nua, const char *call_id)
+{
+  if (nua) {
+#if HAVE_OPEN_C
+    struct nua_stack_handle_by_call_id_args a;
+	a.retval = NULL;
+    a.nua = nua;
+    a.call_id = call_id;
+#else
+    struct nua_stack_handle_by_call_id_args a = { NULL, nua, call_id };
+#endif
+
+    if (su_task_execute(nua->nua_server,
+			nua_stack_handle_by_call_id_call, (void *)&a,
+			NULL) == 0) {
+      nua_handle_t *nh = a.retval;
+
+      if (nh && !NH_IS_DEFAULT(nh) && nh->nh_valid)
+	return nua_handle_ref(nh);
+    }
+  }
+  return NULL;
+}
