@@ -145,7 +145,7 @@ nea_event_t *nh_notifier_event(nua_handle_t *nh,
 
   if (ev == NULL) {
     char *o_type = su_strdup(home, event->o_type);
-    char *o_subtype = strchr(o_type, '.');
+    char *o_subtype = o_type ? strchr(o_type, '.') : NULL;
 
     if (o_subtype)
       *o_subtype++ = '\0';
@@ -262,7 +262,7 @@ void nua_stack_authorize(nua_t *nua,
     nua_stack_event(nua, nh, NULL, e, SIP_200_OK, NULL);
   }
   else {
-    nua_stack_event(nua, nh, NULL, e, NUA_INTERNAL_ERROR, NULL);
+    nua_stack_event(nua, nh, NULL, e, NUA_ERROR_AT(__FILE__, __LINE__), NULL);
   }
 }
 
