@@ -74,8 +74,10 @@ static int nua_notify_usage_add(nua_handle_t *nh,
 				   nua_dialog_state_t *ds,
 				   nua_dialog_usage_t *du);
 static void nua_notify_usage_remove(nua_handle_t *nh, 
-				       nua_dialog_state_t *ds,
-				       nua_dialog_usage_t *du);
+				    nua_dialog_state_t *ds,
+				    nua_dialog_usage_t *du,
+				    nua_client_request_t *cr,
+				    nua_server_request_t *sr);
 static void nua_notify_usage_refresh(nua_handle_t *nh,
 				     nua_dialog_state_t *ds,
 				     nua_dialog_usage_t *du,
@@ -90,6 +92,7 @@ static nua_usage_class const nua_notify_usage[1] = {
     nua_notify_usage_add,
     nua_notify_usage_remove,
     nua_notify_usage_name,
+    nua_base_usage_update_params,
     NULL,
     nua_notify_usage_refresh,
     nua_notify_usage_shutdown,
@@ -102,8 +105,8 @@ static char const *nua_notify_usage_name(nua_dialog_usage_t const *du)
 
 static 
 int nua_notify_usage_add(nua_handle_t *nh, 
-			   nua_dialog_state_t *ds,
-			   nua_dialog_usage_t *du)
+			 nua_dialog_state_t *ds,
+			 nua_dialog_usage_t *du)
 {
   ds->ds_has_events++;
   ds->ds_has_notifys++;
@@ -112,8 +115,10 @@ int nua_notify_usage_add(nua_handle_t *nh,
 
 static 
 void nua_notify_usage_remove(nua_handle_t *nh, 
-			       nua_dialog_state_t *ds,
-			       nua_dialog_usage_t *du)
+			     nua_dialog_state_t *ds,
+			     nua_dialog_usage_t *du,
+			     nua_client_request_t *cr,
+			     nua_server_request_t *sr)
 {
   ds->ds_has_events--;	
   ds->ds_has_notifys--;	
@@ -129,28 +134,6 @@ void nua_notify_usage_remove(nua_handle_t *nh,
  * @b SUBSCRIBE request is used to query SIP event state or establish a SIP
  * event subscription.
  *
- * Initial SUBSCRIBE requests are dropped with <i>489 Bad Event</i>
- * response, unless the application has explicitly included the @Event in
- * the list of allowed events with nua_set_params() tag NUTAG_ALLOW_EVENTS()
- * (or SIPTAG_ALLOW_EVENTS() or SIPTAG_ALLOW_EVENTS_STR()). The application
- * can decide whether to accept the SUBSCRIBE request or reject it. The
- * nua_response() call responding to a SUBSCRIBE request must have
- * NUTAG_WITH() (or NUTAG_WITH_THIS()/NUTAG_WITH_SAVED()) tag.
- *
- * If the application accepts the SUBSCRIBE request, it must immediately
- * send an initial NOTIFY establishing the dialog. This is because the
- * response to the SUBSCRIBE request may be lost because the SUBSCRIBE
- * request was forked by an intermediate proxy. 
- *
- * SUBSCRIBE requests modifying (usually refreshing or terminating) an
- * existing event subscription are accepted by default and a <i>200 OK</i>
- * response along with a copy of previously sent NOTIFY is sent
- * automatically.
- *
- * By default, only event subscriptions accepted are those created
- * implicitly by REFER request. See #nua_i_refer how the application must
- * handle the REFER requests.
- *
  * @param status status code of response sent automatically by stack
  * @param phrase response phrase sent automatically by stack
  * @param nh     operation handle associated with the incoming request
@@ -159,6 +142,46 @@ void nua_notify_usage_remove(nua_handle_t *nh,
  * @param sip    SUBSCRIBE request headers
  * @param tags   NUTAG_SUBSTATE()
  *
+ * Initial SUBSCRIBE requests are dropped with <i>489 Bad Event</i>
+ * response, unless the application has explicitly included the @Event in
+ * the list of allowed events with nua_set_params() tag NUTAG_ALLOW_EVENTS()
+ * (or SIPTAG_ALLOW_EVENTS() or SIPTAG_ALLOW_EVENTS_STR()).
+ *
+ * If the event has been allowed the application
+ * can decide whether to accept the SUBSCRIBE request or reject it. The
+ * nua_response() call responding to a SUBSCRIBE request must have
+ * NUTAG_WITH() (or NUTAG_WITH_THIS()/NUTAG_WITH_SAVED()) tag.
+ *
+ * If the application accepts the SUBSCRIBE request, it must immediately
+ * send an initial NOTIFY establishing the dialog. This is because the
+ * response to the SUBSCRIBE request may be lost by an intermediate proxy
+ * because it had forked the SUBSCRIBE request.
+ *
+ * SUBSCRIBE requests modifying (usually refreshing or terminating) an
+ * existing event subscription are accepted by default and a <i>200 OK</i>
+ * response along with a copy of previously sent NOTIFY is sent
+ * automatically to the subscriber.
+ *
+ * By default, only event subscriptions accepted are those created
+ * implicitly by REFER request. See #nua_i_refer how the application must
+ * handle the REFER requests.
+ *
+ * @par Subscription Lifetime and Terminating Subscriptions
+ *
+ * Accepting the SUBSCRIBE request creates a dialog with a <i>notifier
+ * dialog usage</i> on the handle. The dialog usage is active, until the
+ * subscriber terminates the subscription, it times out or the application
+ * terminates the usage with nua_notify() call containing the tag
+ * NUTAG_SUBSTATE(nua_substate_terminated) or @SubscriptionState header with
+ * state "terminated" and/or expiration time 0. 
+ *
+ * When the subscriber terminates the subscription, the application is
+ * notified of an termination by a #nua_i_subscribe event with
+ * NUTAG_SUBSTATE(nua_substate_terminated) tag. When the subscription times
+ * out, nua automatically initiates a NOTIFY transaction. When it is
+ * terminated, the application is sent a #nua_r_notify event with
+ * NUTAG_SUBSTATE(nua_substate_terminated) tag. 
+ *
  * @sa @RFC3265, nua_notify(), NUTAG_SUBSTATE(), @SubscriptionState,
  * @Event, nua_subscribe(), #nua_r_subscribe, #nua_i_refer, nua_refer()
  *
@@ -219,7 +242,8 @@ int nua_subscribe_server_preprocess(nua_server_request_t *sr)
   sip_event_t *o = sip->sip_event;
   char const *event = o ? o->o_type : NULL;
   /* Maximum expiration time */
-  unsigned long expires = 3600;
+  unsigned long expires = sip->sip_expires ? sip->sip_expires->ex_delta : 3600;
+  sip_time_t now = sip_now();
 
   assert(nh && nh->nh_nua->nua_dhandle != nh);
   
@@ -241,9 +265,10 @@ int nua_subscribe_server_preprocess(nua_server_request_t *sr)
 
   nu = nua_dialog_usage_private(du);
 
-  if (sip->sip_expires && sip->sip_expires->ex_delta < expires)
-    expires = sip->sip_expires->ex_delta;
-  nu->nu_requested = sip_now() + expires;
+  if (now + expires >= now)
+    nu->nu_requested = now + expires;
+  else
+    nu->nu_requested = SIP_TIME_MAX - 1;
 
 #if SU_HAVE_EXPERIMENTAL
   nu->nu_etags = 
@@ -277,9 +302,23 @@ int nua_subscribe_server_respond(nua_server_request_t *sr, tagi_t const *tags)
       sip_time_t now = sip_now();
 
       if (nu->nu_requested) {
-	if (nu->nu_requested > nu->nu_expires)
+	if (sip->sip_expires) {
+	  /* Expires in response can only shorten the expiration time */
+	  if (nu->nu_requested > now + sip->sip_expires->ex_delta) 
+	    nu->nu_requested = now + sip->sip_expires->ex_delta;
+	}
+	else {
+	  unsigned sub_expires = NH_PGET(sr->sr_owner, sub_expires);
+	  if (nu->nu_requested > now + sub_expires)
+	    nu->nu_requested = now + sub_expires;
+	}
+
+	if (nu->nu_requested >= now)
 	  nu->nu_expires = nu->nu_requested;
-	else if (nu->nu_expires <= now || nu->nu_requested <= now)
+	else
+	  nu->nu_expires = now;
+
+	if (nu->nu_expires <= now)
 	  nu->nu_substate = nua_substate_terminated;
       }
 
@@ -287,7 +326,7 @@ int nua_subscribe_server_respond(nua_server_request_t *sr, tagi_t const *tags)
 	ex->ex_delta = nu->nu_expires - now;
     }
     else {
-      /* Add header Expires: 0 */
+      /* Always add header Expires: 0 */
     }
 
     if (!sip->sip_expires || sip->sip_expires->ex_delta > ex->ex_delta)
@@ -302,7 +341,8 @@ int nua_subscribe_server_report(nua_server_request_t *sr, tagi_t const *tags)
 {
   nua_handle_t *nh = sr->sr_owner;
   nua_dialog_state_t *ds = nh->nh_ds;
-  struct notifier_usage *nu = nua_dialog_usage_private(sr->sr_usage);
+  nua_dialog_usage_t *du = sr->sr_usage;
+  struct notifier_usage *nu = nua_dialog_usage_private(du);
   enum nua_substate substate = nua_substate_terminated;
   int notify = 0;
   int retval;
@@ -327,16 +367,18 @@ int nua_subscribe_server_report(nua_server_request_t *sr, tagi_t const *tags)
     else 
 #endif
       notify = 1;
+
+    notify = notify && du->du_cr != NULL;
   }
 
   retval = nua_base_server_treport(sr, NUTAG_SUBSTATE(substate), TAG_END());
 
-  if (retval >= 2 || nu == NULL)
+  if (retval >= 2 || du == NULL)
     return retval;
   
   if (notify) {
     /* Send NOTIFY (and terminate subscription, when needed) */
-    nua_dialog_usage_refresh(nh, ds, sr->sr_usage, sip_now());
+    nua_dialog_usage_refresh(nh, ds, du, sip_now());
   }
   
   return retval;
@@ -351,11 +393,22 @@ int nua_subscribe_server_report(nua_server_request_t *sr, tagi_t const *tags)
  *
  * This function is used when the application implements itself the
  * notifier. The application must provide valid @SubscriptionState and
- * @Event headers using SIP tags. If there is no @SubscriptionState header,
- * the subscription state can be modified with NUTAG_SUBSTATE().
+ * @Event headers using SIP tags. The subscription state can be modified
+ * with NUTAG_SUBSTATE(), however, its effect is overriden by
+ * @SubscriptionState header included in the nua_notify() tags.
  *
  * @bug If the @Event is not given by application, stack uses the @Event
  * header from the first subscription usage on handle.
+ * 
+ * If there is no active <i>notifier dialog usage</i> or no notifier dialog
+ * usage matches the @Event header given by the application the nua_notify()
+ * request is rejected locally by the stack with status code 481. The local
+ * rejection can be bypassed if NUTAG_NEWSUB(1) is included in tags.
+ *
+ * Please note that including NUTAG_NEWSUB(1) in nua_notify() tags if there
+ * is a valid subscription may lead to an extra NOTIFY sent to subscriber if
+ * the subscription had been terminated by the subscriber or by a timeout
+ * before the nua_notify() is processed.
  *
  * @param nh              Pointer to operation handle
  * @param tag, value, ... List of tagged parameters
@@ -365,8 +418,9 @@ int nua_subscribe_server_report(nua_server_request_t *sr, tagi_t const *tags)
  *
  * @par Related Tags:
  *    NUTAG_SUBSTATE() \n
+ *    NUTAG_NEWSUB() \n
  *    Tags of nua_set_hparams() \n
- *    Tags in <sip_tag.h>
+ *    Header tags defined in <sofia-sip/sip_tag.h>
  *
  * @par Events:
  *    #nua_r_notify
@@ -390,20 +444,21 @@ static int nua_notify_client_report(nua_client_request_t *cr,
 				    tagi_t const *tags);
 
 static nua_client_methods_t const nua_notify_client_methods = {
-  SIP_METHOD_NOTIFY,
-  0,
-  { 
+  SIP_METHOD_NOTIFY,		/* crm_method, crm_method_name */
+  0,				/* crm_extra */
+  {				/* crm_flags */
     /* create_dialog */ 1,
     /* in_dialog */ 1,
     /* target refresh */ 1
   },
-  /* nua_notify_client_template */ NULL,
-  nua_notify_client_init,
-  nua_notify_client_request,
-  /* nua_notify_client_check_restart */ NULL,
-  /* nua_notify_client_response */ NULL,
-  /* nua_notify_client_preliminary */ NULL,
-  nua_notify_client_report
+  NULL,				/* crm_template */
+  nua_notify_client_init,	/* crm_init */
+  nua_notify_client_request,	/* crm_send */
+  NULL,				/* crm_check_restart */
+  NULL,				/* crm_recv */
+  NULL,				/* crm_preliminary */
+  nua_notify_client_report,	/* crm_report */
+  NULL,				/* crm_complete */
 };
 
 /**@internal Send NOTIFY. */
@@ -461,14 +516,25 @@ static int nua_notify_client_init(nua_client_request_t *cr,
       if (now + expires < now)
 	expires = SIP_TIME_MAX - now - 1;
 
-      /* Notifier can only shorten the subscription time */ 
-      if (nu->nu_requested == 0 || nu->nu_requested >= now + expires)
+      /* We can change the lifetime of unsolicited subscription at will */
+      if (nu->nu_requested == 0)
+	nu->nu_expires = nu->nu_requested = now + expires;
+      /* Notifier can only shorten the subscribed time */ 
+      else if (nu->nu_requested >= now + expires)
 	nu->nu_expires = nu->nu_requested = now + expires;
     }
+    else {
+      if (nu->nu_requested >= nu->nu_expires)
+	nu->nu_expires = nu->nu_requested;
+    }
+
   }
   else {
     enum nua_substate substate = nu->nu_substate;
 
+    if (nu->nu_requested >= nu->nu_expires)
+      nu->nu_expires = nu->nu_requested;
+
     if (nu->nu_expires > now) {
       tagi_t const *t = tl_find_last(tags, nutag_substate);
       if (t)
@@ -493,9 +559,6 @@ static int nua_notify_client_init(nua_client_request_t *cr,
     }
   }
 
-  if (nu->nu_substate == nua_substate_terminated)
-    cr->cr_terminating = 1;
-
   cr->cr_usage = du;
 
   return nua_notify_client_init_etag(cr, msg, sip, tags);
@@ -635,6 +698,18 @@ int nua_notify_client_request(nua_client_request_t *cr,
   if (nu->nu_substate == nua_substate_terminated)
     cr->cr_terminating = 1;
 
+  if (cr->cr_terminating) {
+    nua_server_request_t *sr;
+    for (sr = du->du_dialog->ds_sr; sr; sr = sr->sr_next) {
+      if (sr->sr_usage == du) {
+	/* If subscribe has not been responded, don't terminate usage by NOTIFY */
+	sr->sr_terminating = 1;
+	cr->cr_terminating = 0;
+	break;
+      }
+    }
+  }
+
   if (du->du_event && !sip->sip_event)
     sip_add_dup(cr->cr_msg, sip, (sip_header_t *)du->du_event);
 
@@ -648,7 +723,12 @@ int nua_notify_client_request(nua_client_request_t *cr,
  * The @b NOTIFY may be sent explicitly by nua_notify() or implicitly by NUA
  * state machine. Implicit @b NOTIFY is sent when an established dialog is
  * refreshed by client or it is terminated (either by client or because of a
- * timeout)
+ * timeout).
+ *
+ * The current subscription state is included in NUTAG_SUBSTATE() tag. The
+ * nua_substate_terminated indicates that the subscription is terminated,
+ * the notifier usage has been removed and when there was no other usages of
+ * the dialog the dialog state is also removed.
  *
  * @param status response status code
  *               (if the request is retried, @a status is 100, the @a
@@ -663,7 +743,7 @@ int nua_notify_client_request(nua_client_request_t *cr,
  * @param tags   NUTAG_SUBSTATE() indicating subscription state
  *               SIPTAG_EVENT() indicating subscription event
  *
- * @sa nua_notify(), @RFC3265, #nua_i_subscribe, #nua_i_refer
+ * @sa nua_notify(), @RFC3265, #nua_i_subscribe, #nua_i_refer, NUTAG_SUBSTATE()
  *
  * @END_NUA_EVENT
  */
@@ -696,7 +776,7 @@ static int nua_notify_client_report(nua_client_request_t *cr,
       nua_client_resend_request(cr, 0);
     }
     else if (nu->nu_expires) {
-      nua_dialog_usage_refresh_at(du, nu->nu_expires);
+      nua_dialog_usage_set_refresh_at(du, nu->nu_expires);
     }
   }
 
@@ -729,11 +809,11 @@ static void nua_notify_usage_refresh(nua_handle_t *nh,
       return;
   }
 
-  nua_stack_tevent(nh->nh_nua, nh, NULL, e, NUA_INTERNAL_ERROR,
+  nua_stack_tevent(nh->nh_nua, nh, NULL, e, NUA_ERROR_AT(__FILE__, __LINE__),
 		   NUTAG_SUBSTATE(nua_substate_terminated),
 		   TAG_END());
 
-  nua_dialog_usage_remove(nh, ds, du);
+  nua_dialog_usage_remove(nh, ds, du, NULL, NULL);
 }
 
 /** @interal Shut down NOTIFY usage. 
@@ -756,12 +836,15 @@ static int nua_notify_usage_shutdown(nua_handle_t *nh,
       return 0;
   }
   else {
-    if (nua_client_create(nh, nua_r_notify, 
-			  &nua_notify_client_methods, NULL) >= 0)
+    if (nua_client_tcreate(nh, nua_r_notify, 
+			   &nua_notify_client_methods, 
+			   SIPTAG_EVENT(du->du_event),
+			   NUTAG_SUBSTATE(nua_substate_terminated),
+			   TAG_END()) >= 0)
       return 0;
   }
 
-  nua_dialog_usage_remove(nh, ds, du);
+  nua_dialog_usage_remove(nh, ds, du, NULL, NULL);
   return 200;
 }
 
@@ -852,7 +935,18 @@ int nua_refer_server_respond(nua_server_request_t *sr, tagi_t const *tags)
 
 /** @NUA_EVENT nua_i_refer
  *
- * Incoming @b REFER request used to transfer calls.
+ * Incoming @b REFER request used to transfer calls. The tag list will
+ * contain tag NUTAG_REFER_EVENT() with the @Event header constructed from
+ * the REFER request. It will also contain the SIPTAG_REFERRED_BY() tag with
+ * the @ReferredBy header containing the identity of the party sending the
+ * REFER. The @ReferredBy structure contained in the tag is constructed from
+ * the @From header if the @ReferredBy header was not present in the REFER
+ * request.
+ * 
+ * The application can let the nua to send NOTIFYs from the call it
+ * initiates with nua_invite() if it includes in the nua_invite() arguments
+ * both the NUTAG_NOTIFY_REFER() with the handle with which nua_i_refer was
+ * received and the NUTAG_REFER_EVENT() from #nua_i_refer event tags.
  *
  * @param status status code of response sent automatically by stack
  * @param phrase a short textual description of @a status code
