@@ -45,10 +45,11 @@
 #include <sofia-sip/su_tag_io.h>
 
 #define SU_ROOT_MAGIC_T   struct nua_s
-#define SU_MSG_ARG_T      struct event_s
+#define SU_MSG_ARG_T      struct nua_ee_data
 #define SU_TIMER_ARG_T    struct nua_client_request
 
 #define NUA_SAVED_EVENT_T su_msg_t *
+#define NUA_SAVED_SIGNAL_T su_msg_t *
 
 #define NTA_AGENT_MAGIC_T    struct nua_s
 #define NTA_LEG_MAGIC_T      struct nua_handle_s
@@ -84,6 +85,26 @@
  *
  * ======================================================================== */
 
+/* ---------------------------------------------------------------------- */
+/* Internal types */
+
+/** Extended event data. */
+typedef struct nua_ee_data {
+  nua_t *ee_nua;
+  nua_event_data_t ee_data[1];
+} nua_ee_data_t;		
+
+/** @internal Linked stack frames from nua event callback */
+struct nua_event_frame_s {
+  nua_event_frame_t *nf_next;
+  nua_saved_event_t nf_saved[1];
+};
+
+
+static void nua_event_deinit(nua_ee_data_t *ee);
+static void nua_application_event(nua_t *, su_msg_r, nua_ee_data_t *ee);
+static void nua_stack_signal(nua_t *nua, su_msg_r, nua_ee_data_t *ee);
+
 nua_handle_t *nh_create(nua_t *nua, tag_type_t t, tag_value_t v, ...);
 static void nh_append(nua_t *nua, nua_handle_t *nh);
 static void nh_remove(nua_t *nua, nua_handle_t *nh);
@@ -188,7 +209,7 @@ int nua_stack_init(su_root_t *root, nua_t *nua)
   if (nua_stack_set_from(nua, 1, nua->nua_args) < 0)
     return -1;
 
-  if (NHP_ISSET(dnh->nh_prefs, detect_network_updates))
+  if (nua->nua_prefs->ngp_detect_network_updates)
     nua_stack_launch_network_change_detector(nua);
 
   nua_stack_timer(nua, nua->nua_timer, NULL);
@@ -272,25 +293,26 @@ int nua_stack_event(nua_t *nua, nua_handle_t *nh, msg_t *msg,
   if ((event > nua_r_authenticate && event <= nua_r_ack)
       || event < nua_i_error
       || (nh && !nh->nh_valid)
-      || (nua->nua_shutdown && event != nua_r_shutdown)) {
+      || (nua->nua_shutdown && event != nua_r_shutdown && 
+	  !nua->nua_prefs->ngp_shutdown_events)) {
     if (msg)
       msg_destroy(msg);
     return event;
   }
 
   if (tags) {
-    e_len = offsetof(event_t, e_tags);
+    e_len = offsetof(nua_ee_data_t, ee_data[0].e_tags);
     len = tl_len(tags);
     xtra = tl_xtra(tags, len);
   }
   else {
-    e_len = sizeof(event_t), len = 0, xtra = 0;
+    e_len = sizeof(nua_ee_data_t), len = 0, xtra = 0;
   }
   p_len = phrase ? strlen(phrase) + 1 : 1;
 
-  if (su_msg_create(sumsg, nua->nua_client, su_task_null,
-		    nua_event, e_len + len + xtra + p_len) == 0) {
-    event_t *e = su_msg_data(sumsg);
+  if (su_msg_new(sumsg, e_len + len + xtra + p_len) == 0) {
+    nua_ee_data_t *ee = su_msg_data(sumsg);
+    nua_event_data_t *e = ee->ee_data;
     void *p;
 
     if (tags) {
@@ -303,21 +325,138 @@ int nua_stack_event(nua_t *nua, nua_handle_t *nh, msg_t *msg,
     else
       p = e + 1;
 
+    ee->ee_nua = nua_stack_ref(nua);
     e->e_event = event;
-    e->e_nh = nh ? nua_handle_ref(nh) : nua->nua_dhandle;
+    e->e_nh = nh ? nua_handle_ref(nh) : NULL;
     e->e_status = status;
     e->e_phrase = strcpy(p, phrase ? phrase : "");
     if (msg)
       e->e_msg = msg, su_home_threadsafe(msg_home(msg));
 
-    if (su_msg_send(sumsg) != 0 && nh)
-      nua_handle_unref(nh);
+    su_msg_deinitializer(sumsg, nua_event_deinit);
+
+    su_msg_send_to(sumsg, nua->nua_client, nua_application_event);
   }
 
   return event;
 }
 
-/* ----------------------------------------------------------------------
+static
+void nua_event_deinit(nua_ee_data_t *ee)
+{
+  nua_t *nua = ee->ee_nua;
+  nua_event_data_t *e = ee->ee_data;
+  nua_handle_t *nh = e->e_nh;
+
+  if (e->e_msg)
+    msg_destroy(e->e_msg), e->e_msg = NULL;
+
+  if (nh)
+    nua_handle_unref(nh), e->e_nh = NULL;
+
+  if (nua)
+    nua_stack_unref(nua), ee->ee_nua = NULL;
+}
+
+/*# Receive event from protocol machine and hand it over to application */
+static
+void nua_application_event(nua_t *dummy, su_msg_r sumsg, nua_ee_data_t *ee)
+{
+  nua_t *nua = ee->ee_nua;
+  nua_event_data_t *e = ee->ee_data;
+  nua_handle_t *nh = e->e_nh;
+
+  enter;
+
+  ee->ee_nua = NULL;
+  e->e_nh = NULL;
+
+  if (nh == NULL) {
+    /* Xyzzy */
+  }
+  else if (nh->nh_valid) {
+    if (!nh->nh_ref_by_user) {
+      /* Application must now call nua_handle_destroy() */
+      nh->nh_ref_by_user = 1;
+      nua_handle_ref(nh);	
+    }
+  }
+  else if (!nh->nh_valid) {	/* Handle has been destroyed */
+    if (nua_log->log_level >= 7) {
+      char const *name = nua_event_name(e->e_event) + 4;
+      SU_DEBUG_7(("nua(%p): event %s dropped\n", (void *)nh, name));
+    }
+    nua_handle_unref(nh);
+    nua_stack_unref(nua);
+    return;
+  }
+
+  if (e->e_event == nua_r_shutdown && e->e_status >= 200)
+    nua->nua_shutdown_final = 1;
+
+  if (nua->nua_callback) {
+    nua_event_frame_t frame[1];
+    
+    su_msg_save(frame->nf_saved, sumsg);
+    frame->nf_next = nua->nua_current, nua->nua_current = frame;
+
+    nua->nua_callback(e->e_event, e->e_status, e->e_phrase,
+		      nua, nua->nua_magic,
+		      nh, nh ? nh->nh_magic : NULL,
+		      e->e_msg ? sip_object(e->e_msg) : NULL,
+		      e->e_tags);
+
+    su_msg_destroy(frame->nf_saved);
+    nua->nua_current = frame->nf_next;
+  }
+
+  nua_handle_unref(nh);
+  nua_stack_unref(nua);
+}
+
+/** Get current request message. @NEW_1_12_4.
+ *
+ * @note A response message is returned when processing response message.
+ *
+ * @sa #nua_event_e, nua_respond(), NUTAG_WITH_CURRENT()
+ */
+msg_t *nua_current_request(nua_t const *nua)
+{
+  if (nua && nua->nua_current && su_msg_is_non_null(nua->nua_current->nf_saved))
+    return su_msg_data(nua->nua_current->nf_saved)->ee_data->e_msg;
+  return NULL;
+}
+
+/** Get request message from saved nua event. @NEW_1_12_4. 
+ *
+ * @sa nua_save_event(), nua_respond(), NUTAG_WITH_SAVED(), 
+ */
+msg_t *nua_saved_event_request(nua_saved_event_t const *saved)
+{
+  return saved && saved[0] ? su_msg_data(saved)->ee_data->e_msg : NULL;
+}
+
+/** Save nua event and its arguments.
+ *
+ * @sa #nua_event_e, nua_event_data() nua_saved_event_request(), nua_destroy_event()
+ */
+int nua_save_event(nua_t *nua, nua_saved_event_t return_saved[1])
+{
+  if (return_saved) {
+    if (nua && nua->nua_current) {
+      su_msg_save(return_saved, nua->nua_current->nf_saved);
+      return su_msg_is_non_null(return_saved);
+    }
+    else
+      *return_saved = NULL;
+  }
+
+  return 0;
+}
+
+/* ---------------------------------------------------------------------- */
+
+/** @internal
  * Post signal to stack itself
  */
 void nua_stack_post_signal(nua_handle_t *nh, nua_event_t event,
@@ -325,28 +464,85 @@ void nua_stack_post_signal(nua_handle_t *nh, nua_event_t event,
 {
   ta_list ta;
   ta_start(ta, tag, value);
-  nua_signal((nh)->nh_nua, nh, NULL, 1, event, 0, NULL, ta_tags(ta));
+  nua_signal((nh)->nh_nua, nh, NULL, event, 0, NULL, ta_tags(ta));
   ta_end(ta);
 }
 
 
+/*# Send a request to the protocol thread */
+int nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg,
+	       nua_event_t event,
+	       int status, char const *phrase,
+	       tag_type_t tag, tag_value_t value, ...)
+{
+  su_msg_r sumsg = SU_MSG_R_INIT;
+  size_t len, xtra, ee_len, l_len = 0, l_xtra = 0;
+  ta_list ta;
+  int retval = -1;
+
+  if (nua == NULL)
+    return -1;
+
+  if (nua->nua_shutdown_started && event != nua_r_shutdown)
+    return -1;
+
+  ta_start(ta, tag, value);
+
+  ee_len = offsetof(nua_ee_data_t, ee_data[0].e_tags);
+  len = tl_len(ta_args(ta));
+  xtra = tl_xtra(ta_args(ta), len);
+
+  if (su_msg_new(sumsg, ee_len + len + l_len + xtra + l_xtra) == 0) {
+    nua_ee_data_t *ee = su_msg_data(sumsg);
+    nua_event_data_t *e = ee->ee_data;
+    tagi_t *t = e->e_tags;
+    void *b = (char *)t + len + l_len;
+
+    tagi_t *tend = (tagi_t *)b;
+    char *bend = (char *)b + xtra + l_xtra;
+
+    t = tl_dup(t, ta_args(ta), &b);
+
+    assert(tend == t); (void)tend; assert(b == bend); (void)bend;
+
+    e->e_always = event == nua_r_destroy || event == nua_r_shutdown;
+    e->e_event = event;
+    e->e_nh = nh ? nua_handle_ref(nh) : NULL;
+    e->e_status = status;
+    e->e_phrase = phrase;
+
+    su_msg_deinitializer(sumsg, nua_event_deinit);
+      
+    retval = su_msg_send_to(sumsg, nua->nua_server, nua_stack_signal);
+
+    if (retval == 0){
+      SU_DEBUG_7(("nua(%p): %s signal %s\n", (void *)nh,
+		  "sent", nua_event_name(event) + 4));
+    }
+    else {
+      SU_DEBUG_0(("nua(%p): %s signal %s\n", (void *)nh,
+		  "FAILED TO SEND", nua_event_name(event) + 4));
+
+    }
+  }
+
+  ta_end(ta);
+
+  return retval;
+}
+
 /* ----------------------------------------------------------------------
  * Receiving events from client
  */
-void nua_stack_signal(nua_t *nua, su_msg_r msg, nua_event_data_t *e)
+static
+void nua_stack_signal(nua_t *nua, su_msg_r msg, nua_ee_data_t *ee)
 {
+  nua_event_data_t *e = ee->ee_data;
   nua_handle_t *nh = e->e_nh;
   tagi_t *tags = e->e_tags;
   nua_event_t event;
   int error = 0;
 
-  assert(tags);
-
-  if (nua_log->log_level >= 7) {
-    char const *name = nua_event_name(e->e_event) + 4;
-    SU_DEBUG_7(("nua(%p): recv %s\n", (void *)nh, name));
-  }
-
   if (nh) {
     if (!nh->nh_prev)
       nh_append(nua, nh);
@@ -359,10 +555,11 @@ void nua_stack_signal(nua_t *nua, su_msg_r msg, nua_event_data_t *e)
 
   if (nua_log->log_level >= 5) {
     char const *name = nua_event_name(e->e_event);
+
     if (e->e_status == 0)
-      SU_DEBUG_5(("nua(%p): signal %s\n", (void *)nh, name + 4));
+      SU_DEBUG_5(("nua(%p): %s signal %s\n", (void *)nh, "recv", name + 4));
     else
-      SU_DEBUG_5(("nua(%p): signal %s %u %s\n",
+      SU_DEBUG_5(("nua(%p): recv signal %s %u %s\n",
 		  (void *)nh, name + 4,
 		  e->e_status, e->e_phrase ? e->e_phrase : ""));
   }
@@ -459,14 +656,48 @@ void nua_stack_signal(nua_t *nua, su_msg_r msg, nua_event_data_t *e)
   }
 
   if (error < 0) {
-    nua_stack_event(nh->nh_nua, nh, NULL, event, NUA_INTERNAL_ERROR, NULL);
+    nua_stack_event(nh->nh_nua, nh, NULL, event,
+		    NUA_ERROR_AT(__FILE__, __LINE__), NULL);
   }
 
-  if (su_msg_is_non_null(nua->nua_signal))
-    su_msg_destroy(nua->nua_signal);
+  su_msg_destroy(nua->nua_signal);
+}
 
-  if (nh != nua->nua_dhandle)
-    nua_handle_unref(nh);
+/* ====================================================================== */
+/* Signal and event handling */
+
+/** Get event data.
+ *
+ * @sa #nua_event_e, nua_event_save(), nua_saved_event_request(), nua_destroy_event().
+ */
+nua_event_data_t const *nua_event_data(nua_saved_event_t const saved[1])
+{
+  return saved && saved[0] ? su_msg_data(saved)->ee_data : NULL;
+}
+
+/** Destroy saved event.
+ *
+ * @sa #nua_event_e, nua_event_save(), nua_event_data(), nua_saved_event_request().
+ */
+void nua_destroy_event(nua_saved_event_t saved[1])
+{
+  if (saved) su_msg_destroy(saved);
+}
+
+/** @internal Move signal. */
+void nua_move_signal(nua_saved_signal_t a[1], nua_saved_signal_t b[1])
+{
+  su_msg_save(a, b);
+}
+
+void nua_destroy_signal(nua_saved_signal_t saved[1])
+{
+  if (saved) su_msg_destroy(saved);
+}
+
+nua_signal_data_t const *nua_signal_data(nua_saved_signal_t const saved[1])
+{
+  return nua_event_data(saved);
 }
 
 /* ====================================================================== */
@@ -636,7 +867,7 @@ void nua_stack_shutdown(nua_t *nua)
     for (nh = nua->nua_handles; nh; nh = nh_next) {
       nh_next = nh->nh_next;
       while (nh->nh_ds && nh->nh_ds->ds_usage) {
-	nua_dialog_usage_remove(nh, nh->nh_ds, nh->nh_ds->ds_usage);
+	nua_dialog_usage_remove(nh, nh->nh_ds, nh->nh_ds->ds_usage, NULL, NULL);
       }
     }
     su_timer_destroy(nua->nua_timer), nua->nua_timer = NULL;
@@ -810,7 +1041,7 @@ nua_handle_t *nua_stack_incoming_handle(nua_t *nua,
 		 SIPTAG_FROM(from), /* Remote AoR */
 		 TAG_END());
 
-  if (nua_stack_init_handle(nh->nh_nua, nh, NULL) < 0)
+  if (nh && nua_stack_init_handle(nua, nh, NULL) < 0)
     nh_destroy(nua, nh), nh = NULL;
 
   if (nh && create_dialog) {
@@ -890,6 +1121,15 @@ nua_handle_t *nua_stack_handle_by_replaces(nua_t *nua,
   return NULL;
 }
 
+nua_handle_t *nua_stack_handle_by_call_id(nua_t *nua, const char *call_id)
+{
+  if (nua) {
+    nta_leg_t *leg = nta_leg_by_call_id(nua->nua_nta, call_id);
+    if (leg)
+      return nta_leg_magic(leg, nua_stack_process_request);
+  }
+  return NULL;
+}
 
 /** @internal Add authorization data */
 int nh_authorize(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
@@ -990,9 +1230,9 @@ nua_stack_authenticate(nua_t *nua, nua_handle_t *nh, nua_event_t e,
     cr->cr_waiting = cr->cr_wait_for_cred = 0;
     
     if (status < 0)
-      nua_client_response(cr, 900, "Cannot add credentials", NULL);
+      nua_client_response(cr, 900, "Operation cannot add credentials", NULL);
     else
-      nua_client_response(cr, 904, "No matching challenge", NULL);
+      nua_client_response(cr, 904, "Operation has no matching challenge ", NULL);
   }
   else if (status < 0) {
     nua_stack_event(nua, nh, NULL, e, 900, "Cannot add credentials", NULL);
@@ -1113,6 +1353,8 @@ int nua_stack_process_request(nua_handle_t *nh,
        (Call/Transaction Does Not Exist) status code and pass that to the
        server transaction.
     */
+    if (method == sip_method_info)
+      /* accept out-of-dialog info */; else
     if (method != sip_method_message || !NH_PGET(nh, win_messenger_enable))
       sm = NULL;
   }
@@ -1241,11 +1483,12 @@ void nua_server_request_destroy(nua_server_request_t *sr)
   if (sr == NULL)
     return;
 
+  if (SR_HAS_SAVED_SIGNAL(sr))
+    nua_destroy_signal(sr->sr_signal);
+
   if (sr->sr_irq)
     nta_incoming_destroy(sr->sr_irq), sr->sr_irq = NULL;
 
-  su_msg_destroy(sr->sr_signal);
-
   if (sr->sr_request.msg)
     msg_destroy(sr->sr_request.msg), sr->sr_request.msg = NULL;
 
@@ -1378,8 +1621,9 @@ nua_stack_respond(nua_t *nua, nua_handle_t *nh,
     sr->sr_application = status;
     if (tags && nua_stack_set_params(nua, nh, nua_i_none, tags) < 0)
       SR_STATUS1(sr, SIP_500_INTERNAL_SERVER_ERROR);
-    else
+    else {
       sr->sr_status = status, sr->sr_phrase = phrase;
+    }
   }
 
   nua_server_params(sr, tags);
@@ -1478,8 +1722,12 @@ int nua_server_respond(nua_server_request_t *sr, tagi_t const *tags)
 	   sip_add_dup(msg, sip, (void *)NH_PGET(nh, allow)) < 0)
     ;
   else if (!sip->sip_allow_events && 
-	   (method == sip_method_publish || method == sip_method_subscribe) &&
 	   NH_PGET(nh, allow_events) &&
+	   (method == sip_method_publish || method == sip_method_subscribe ||
+	    method == sip_method_options || method == sip_method_refer || 
+	    (sr->sr_initial &&
+	     (method == sip_method_invite ||
+	      method == sip_method_notify))) &&
 	   sip_add_dup(msg, sip, (void *)NH_PGET(nh, allow_events)) < 0)
     ;
   else if (!sip->sip_contact && sr->sr_status < 300 && sr->sr_add_contact &&
@@ -1610,8 +1858,6 @@ int nua_base_server_report(nua_server_request_t *sr, tagi_t const *tags)
   int terminated;
   int handle_can_be_terminated = initial && !sr->sr_event;
 
-  assert(nh);
-
   if (sr->sr_application) {
     /* There was an error sending response */
     if (sr->sr_application != sr->sr_status)
@@ -1638,14 +1884,14 @@ int nua_base_server_report(nua_server_request_t *sr, tagi_t const *tags)
   else
     terminated = sip_response_terminates_dialog(status, sr->sr_method, NULL);
 
+  if (usage && terminated)
+    nua_dialog_usage_remove(nh, nh->nh_ds, usage, NULL, sr);
+
   nua_server_request_destroy(sr);
 
   if (!terminated)
     return 1;
 
-  if (usage)
-    nua_dialog_usage_remove(nh, nh->nh_ds, usage);
-
   if (!initial) {
     if (terminated > 0)
       return 2;
@@ -1756,11 +2002,10 @@ int nua_client_create(nua_handle_t *nh,
     return nua_stack_event(nh->nh_nua, nh, 
 			   NULL,
 			   event,
-			   NUA_INTERNAL_ERROR,
+			   NUA_ERROR_AT(__FILE__, __LINE__),
 			   NULL);
   }
 
-  cr->cr_owner = nh;
   cr->cr_methods = methods;
   cr->cr_event = event;
   cr->cr_method = method;
@@ -1770,17 +2015,25 @@ int nua_client_create(nua_handle_t *nh,
   cr->cr_auto = 1;
 
   if (su_msg_is_non_null(nh->nh_nua->nua_signal)) {
-    nua_event_data_t const *e = su_msg_data(nh->nh_nua->nua_signal);
+    nua_event_data_t *e = su_msg_data(nh->nh_nua->nua_signal)->ee_data;
 
     if (tags == e->e_tags && event == e->e_event) {
       cr->cr_auto = 0;
+
       if (tags) {
-	su_msg_save(cr->cr_signal, nh->nh_nua->nua_signal);
-	cr->cr_tags = tags;
+	nua_move_signal(cr->cr_signal, nh->nh_nua->nua_signal);
+	if (cr->cr_signal) {
+	  /* Steal reference from signal */
+	  cr->cr_owner = e->e_nh, e->e_nh = NULL;
+	  cr->cr_tags = tags;
+	}
       }
     }
   }
 
+  if (cr->cr_owner == NULL)
+    cr->cr_owner = nua_handle_ref(nh);
+
   if (tags && cr->cr_tags == NULL)
     cr->cr_tags = tl_tlist(nh->nh_home, TAG_NEXT(tags));
 
@@ -1847,6 +2100,13 @@ nua_client_request_t *nua_client_request_remove(nua_client_request_t *cr)
   return cr;
 }
 
+void nua_client_request_complete(nua_client_request_t *cr)
+{
+  nua_client_request_remove(cr);
+  if (cr && cr->cr_methods->crm_complete)
+    cr->cr_methods->crm_complete(cr);
+}
+
 void nua_client_request_destroy(nua_client_request_t *cr)
 {
   nua_handle_t *nh;
@@ -1854,14 +2114,12 @@ void nua_client_request_destroy(nua_client_request_t *cr)
   if (cr == NULL)
     return;
 
-  if (cr->cr_methods->crm_deinit)
-    cr->cr_methods->crm_deinit(cr);
+  nua_client_request_complete(cr);
 
   nh = cr->cr_owner;
 
-  su_msg_destroy(cr->cr_signal);
+  nua_destroy_signal(cr->cr_signal);
 
-  nua_client_request_remove(cr);
   nua_client_bind(cr, NULL);
   
   if (cr->cr_msg)
@@ -1870,7 +2128,6 @@ void nua_client_request_destroy(nua_client_request_t *cr)
 
   if (cr->cr_orq)
     nta_outgoing_destroy(cr->cr_orq);
-
   cr->cr_orq = NULL;
 
   if (cr->cr_timer)
@@ -1880,6 +2137,8 @@ void nua_client_request_destroy(nua_client_request_t *cr)
     su_free(nh->nh_home, cr->cr_target);
 
   su_free(nh->nh_home, cr);
+
+  nua_handle_unref(nh);
 }
 
 /** Bind client request to a dialog usage */ 
@@ -1897,7 +2156,9 @@ int nua_client_bind(nua_client_request_t *cr, nua_dialog_usage_t *du)
   }
 
   if (du->du_cr && cr != du->du_cr) {
-    assert(!nua_client_is_queued(du->du_cr));
+    /* This should never happen (but it does):
+       assert(!nua_client_is_queued(du->du_cr)); 
+    */
     if (nua_client_is_queued(du->du_cr))
       return -1;
     if (nua_client_is_reporting(du->du_cr)) {
@@ -1933,7 +2194,7 @@ int nua_client_init_request(nua_client_request_t *cr)
   int error = 0;
   
   if (!cr->cr_method_name)
-    return nua_client_return(cr, NUA_INTERNAL_ERROR, NULL);
+    return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), NULL);
 
   if (cr->cr_msg)
     return nua_client_request_try(cr);
@@ -1958,12 +2219,22 @@ int nua_client_init_request(nua_client_request_t *cr)
   }
 
   if (!cr->cr_methods->crm_template ||
-      !cr->cr_methods->crm_template(cr, &msg, cr->cr_tags))
-    msg = nta_msg_create(nua->nua_nta, 0);
+      cr->cr_methods->crm_template(cr, &msg, cr->cr_tags) == 0)
+    msg = nua_client_request_template(cr);
 
   sip = sip_object(msg);
   if (!sip)
-    return nua_client_return(cr, NUA_INTERNAL_ERROR, msg);
+    return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
+
+  if (nh->nh_tags) {
+    for (t = nh->nh_tags; t; t = t_next(t)) {
+      if (t->t_tag == siptag_contact ||
+	  t->t_tag == siptag_contact_str)
+	has_contact = 1;
+      else if (t->t_tag == nutag_url)
+	url = (url_string_t const *)t->t_value;
+    }
+  }
 
   /**@par Populating SIP Request Message with Tagged Arguments
    *
@@ -1983,30 +2254,6 @@ int nua_client_init_request(nua_client_request_t *cr)
    * However, if a tag value is #SIP_NONE (-1 casted as a void pointer),
    * the values from previous tags are ignored.
    */
-  if (nh->nh_tags) {
-    for (t = nh->nh_tags; t; t = t_next(t)) {
-      if (t->t_tag == siptag_contact ||
-	  t->t_tag == siptag_contact_str)
-	has_contact = 1;
-      else if (t->t_tag == nutag_url)
-	url = (url_string_t const *)t->t_value;
-    }
-
-    t = nh->nh_tags;
-
-    /* Use the From header from the dialog */
-    if (ds->ds_leg && t->t_tag == siptag_from)
-      t++;
-
-    sip_add_tagis(msg, sip, &t);
-  }
-
-  if (!ds->ds_route) {
-    sip_route_t *initial_route = NH_PGET(nh, initial_route);
-    if (initial_route)
-      sip_add_dup(msg, sip, (sip_header_t *)initial_route);
-  }
-
   for (t = cr->cr_tags; t; t = t_next(t)) {
     if (t->t_tag == siptag_contact ||
 	t->t_tag == siptag_contact_str)
@@ -2029,7 +2276,7 @@ int nua_client_init_request(nua_client_request_t *cr)
   
   if ((t = cr->cr_tags)) {
     if (sip_add_tagis(msg, sip, &t) < 0)
-      goto error;
+      return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
   }
 
   /**
@@ -2049,11 +2296,17 @@ int nua_client_init_request(nua_client_request_t *cr)
   if (!ds->ds_leg) {
     if (ds->ds_remote_tag && ds->ds_remote_tag[0] && 
 	sip_to_tag(nh->nh_home, sip->sip_to, ds->ds_remote_tag) < 0)
-      goto error;
+      return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
 
     if (sip->sip_from == NULL && 
 	sip_add_dup(msg, sip, (sip_header_t *)nua->nua_from) < 0)
-      goto error;
+      return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
+
+    if (sip->sip_to == NULL && cr->cr_method == sip_method_register &&
+      sip_add_dup_as(msg, sip, sip_to_class,
+		     (sip_header_t *)sip->sip_from) < 0) {
+      return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
+    }
 
     if (cr->cr_dialog) {
       ds->ds_leg = nta_leg_tcreate(nua->nua_nta,
@@ -2064,12 +2317,12 @@ int nua_client_init_request(nua_client_request_t *cr)
 				   SIPTAG_CSEQ(sip->sip_cseq),
 				   TAG_END());
       if (!ds->ds_leg)
-	goto error;
+	return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
 
       if (!sip->sip_from->a_tag &&
 	  sip_from_tag(msg_home(msg), sip->sip_from,
 		       nta_leg_tag(ds->ds_leg, NULL)) < 0)
-	goto error;
+	return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
     }
   }
   else {
@@ -2078,7 +2331,7 @@ int nua_client_init_request(nua_client_request_t *cr)
   }
 
   if (url && nua_client_set_target(cr, (url_t *)url) < 0)
-    goto error;
+    return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
 
   cr->cr_has_contact = has_contact;
 
@@ -2087,7 +2340,7 @@ int nua_client_init_request(nua_client_request_t *cr)
     if (error < -1)
       msg = NULL;
     if (error < 0)
-      goto error;
+      return nua_client_return(cr, NUA_ERROR_AT(__FILE__, __LINE__), msg);
     if (error != 0)
       return error;
   }
@@ -2097,14 +2350,40 @@ int nua_client_init_request(nua_client_request_t *cr)
 
   return nua_client_request_try(cr);
 
- error:
-  return nua_client_return(cr, NUA_INTERNAL_ERROR, msg);
+}
+
+msg_t *nua_client_request_template(nua_client_request_t *cr)
+{
+  nua_handle_t *nh = cr->cr_owner;
+  nua_t *nua = nh->nh_nua;
+  nua_dialog_state_t *ds = nh->nh_ds;
+
+  msg_t *msg = nta_msg_create(nua->nua_nta, 0);
+  sip_t *sip = sip_object(msg);
+
+  if (!sip)
+    return NULL;
+
+  if (nh->nh_tags) {
+    tagi_t const *t = nh->nh_tags;
+
+    /* Use the From header from the dialog. 
+       From is always first tag in the handle */
+    if (ds->ds_leg && t->t_tag == siptag_from)
+      t++;
+
+    /* When the INVITE message (or any other SIP message) is
+     * created, the tagged values saved with nua_handle() are used first. */
+    sip_add_tagis(msg, sip, &t);
+  }
+
+  return msg;
 }
 
 
 /** Restart the request message.
  *
- * A restarted request has not completed successfully.
+ * A restarted request has not been completed successfully.
  *
  * @retval 0 if request is pending
  * @retval >=1 if error event has been sent
@@ -2124,6 +2403,7 @@ int nua_client_restart_request(nua_client_request_t *cr,
 
     return nua_client_request_try(cr);
   }
+
   return 0;
 }
 
@@ -2187,7 +2467,7 @@ int nua_client_request_try(nua_client_request_t *cr)
   }
 
   if (error < 0)
-    error = nua_client_response(cr, NUA_INTERNAL_ERROR, NULL);
+    error = nua_client_response(cr, NUA_ERROR_AT(__FILE__, __LINE__), NULL);
 
   assert(error > 0);
   return error;
@@ -2225,6 +2505,22 @@ int nua_client_request_sendmsg(nua_client_request_t *cr, msg_t *msg, sip_t *sip)
     while (sip->sip_route)
       sip_route_remove(msg, sip);
   }
+  else if (!ds->ds_route) {
+    sip_route_t *initial_route = NH_PGET(nh, initial_route);
+
+    if (initial_route) {
+      initial_route = sip_route_dup(msg_home(msg), initial_route);
+      if (!initial_route) return -1;
+      msg_header_prepend(msg, (msg_pub_t*)sip, 
+			 /* This should be 
+			    (msg_header_t **)&sip->sip_route
+			  * but directly casting pointer &sip->sip_route gives
+			  * spurious type-punning warning */
+			 (msg_header_t **)((char *)sip + offsetof(sip_t, sip_route)),
+			 (msg_header_t *)initial_route);
+    }
+  }
+
   
   /**
    * For in-dialog requests, the request URI is taken from the @Contact
@@ -2249,8 +2545,9 @@ int nua_client_request_sendmsg(nua_client_request_t *cr, msg_t *msg, sip_t *sip)
 
   /**
    * Next, values previously set with nua_set_params() or nua_set_hparams()
-   * are used: @Allow, @Supported, @Organization, and @UserAgent headers are
-   * added to the request if they are not already set. 
+   * are used: @Allow, @Supported, @Organization, @UserAgent and
+   * @AllowEvents headers are added to the request if they are not already
+   * set.
    */
   if (!sip->sip_allow)
     sip_add_dup(msg, sip, (sip_header_t*)NH_PGET(nh, allow));
@@ -2269,6 +2566,20 @@ int nua_client_request_sendmsg(nua_client_request_t *cr, msg_t *msg, sip_t *sip)
   if (!sip->sip_user_agent && NH_PGET(nh, user_agent))
     sip_add_make(msg, sip, sip_user_agent_class, NH_PGET(nh, user_agent));
 
+  /** Any node implementing one or more event packages SHOULD include an
+   * appropriate @AllowEvents header indicating all supported events in
+   * all methods which initiate dialogs and their responses (such as
+   * INVITE) and OPTIONS responses.
+   */
+  if (!sip->sip_allow_events &&
+      NH_PGET(nh, allow_events) &&
+      (method == sip_method_notify || /* Always in NOTIFY */
+       (!ds->ds_remote_tag &&	      /* And in initial requests */
+	(method == sip_method_subscribe || method == sip_method_refer ||
+	 method == sip_method_options ||
+	 method == sip_method_invite))))
+    sip_add_dup(msg, sip, (void *)NH_PGET(nh, allow_events));
+
   /**
    * Next, the stack generates a @Contact header for the request (unless
    * the application already gave a @Contact header or it does not want to
@@ -2341,6 +2652,8 @@ int nua_base_client_request(nua_client_request_t *cr, msg_t *msg, sip_t *sip,
 			    tagi_t const *tags)
 {
   nua_handle_t *nh = cr->cr_owner;
+  int proxy_is_set = NH_PISSET(nh, proxy);
+  url_string_t * proxy = NH_PGET(nh, proxy);
 
   if (nh->nh_auth) {
     if (cr->cr_challenged || 
@@ -2356,6 +2669,8 @@ int nua_base_client_request(nua_client_request_t *cr, msg_t *msg, sip_t *sip,
 				    nua_client_orq_response, cr,
 				    NULL, 
 				    msg,
+				    TAG_IF(proxy_is_set,
+					   NTATAG_DEFAULT_PROXY(proxy)),
 				    TAG_NEXT(tags));
 
   return cr->cr_orq ? 0 : -1;
@@ -2426,6 +2741,7 @@ int nua_client_response(nua_client_request_t *cr,
     return 0;
 
   cr->cr_status = status;
+  cr->cr_phrase = phrase;
 
   if (status < 200) {
     /* Xyzzy */
@@ -2471,6 +2787,7 @@ int nua_client_response(nua_client_request_t *cr,
       cr->cr_methods->crm_preliminary(cr, status, phrase, sip);
     else
       nua_base_client_response(cr, status, phrase, sip, NULL);
+    cr->cr_phrase = NULL;
     return 0;
   }  
 
@@ -2490,10 +2807,12 @@ int nua_client_check_restart(nua_client_request_t *cr,
 			     char const *phrase,
 			     sip_t const *sip)
 {
-  nua_handle_t *nh = cr->cr_owner;
+  nua_handle_t *nh;
 
   assert(cr && status >= 200 && phrase && sip);
 
+  nh = cr->cr_owner;
+
   if (cr->cr_retry_count > NH_PGET(nh, retry_count))
     return 0;
 
@@ -2519,14 +2838,16 @@ int nua_base_client_check_restart(nua_client_request_t *cr,
 
     switch (status) {
     case 302:
-      if (nua_client_set_target(cr, sip->sip_contact->m_url) >= 0)
+      if (nua_dialog_zap(nh, nh->nh_ds) == 0 &&
+	  nua_client_set_target(cr, sip->sip_contact->m_url) >= 0)
 	return nua_client_restart(cr, 100, "Redirected");
       break;
 
     case 305:
       sip_route_init(r);
       *r->r_url = *sip->sip_contact->m_url;
-      if (sip_add_dup(cr->cr_msg, cr->cr_sip, (sip_header_t *)r) >= 0)
+      if (nua_dialog_zap(nh, nh->nh_ds) == 0 &&
+	  sip_add_dup(cr->cr_msg, cr->cr_sip, (sip_header_t *)r) >= 0)
 	return nua_client_restart(cr, 100, "Redirected via a proxy");
       break;
 
@@ -2574,9 +2895,11 @@ int nua_base_client_check_restart(nua_client_request_t *cr,
 
       cr->cr_challenged = 1;
 
-      if (invalid)
+      if (invalid) {
 	/* Bad username/password */
+	SU_DEBUG_7(("nua(%p): bad credentials, clearing them\n", (void *)nh));
 	auc_clear_credentials(&nh->nh_auth, NULL, NULL);
+      }
       else if (auc_has_authorization(&nh->nh_auth)) 
 	return nua_client_restart(cr, 100, "Request Authorized by Cache");
 
@@ -2585,6 +2908,7 @@ int nua_base_client_check_restart(nua_client_request_t *cr,
       cr->cr_waiting = cr->cr_wait_for_cred = 1;
       nua_client_report(cr, status, phrase, NULL, orq, NULL);
       nta_outgoing_destroy(orq);
+      cr->cr_status = 0, cr->cr_phrase = NULL;
 
       return 1;
     }
@@ -2593,7 +2917,7 @@ int nua_base_client_check_restart(nua_client_request_t *cr,
   if (500 <= status && status < 600 && 
       sip->sip_retry_after && 
       sip->sip_retry_after->af_delta < 32) {
-    char phrase[18];		/* Retry-After: XXXX\0 */
+    char phrase[18];		/* Retry After XXXX\0 */
 
     if (cr->cr_timer == NULL)
       cr->cr_timer = su_timer_create(su_root_task(nh->nh_nua->nua_root), 0);
@@ -2609,6 +2933,7 @@ int nua_base_client_check_restart(nua_client_request_t *cr,
     cr->cr_waiting = cr->cr_wait_for_timer = 1;
     nua_client_report(cr, 100, phrase, NULL, orq, NULL);
     nta_outgoing_destroy(orq);
+    cr->cr_status = 0, cr->cr_phrase = NULL;
     return 1;
   }
 
@@ -2637,10 +2962,11 @@ int nua_client_restart(nua_client_request_t *cr,
 		       int status, char const *phrase)
 {
   nua_handle_t *nh = cr->cr_owner;
+  nua_dialog_state_t *ds = nh->nh_ds;
   nta_outgoing_t *orq;
   int error = -1, terminated, graceful;
-  msg_t *msg;
-  sip_t *sip;
+  msg_t *msg = NULL;
+  sip_t *sip = NULL;
 
   if (cr->cr_retry_count > NH_PGET(nh, retry_count))
     return 0;
@@ -2648,9 +2974,24 @@ int nua_client_restart(nua_client_request_t *cr,
   orq = cr->cr_orq, cr->cr_orq = NULL;  assert(orq);
   terminated = cr->cr_terminated, cr->cr_terminated = 0;
   graceful = cr->cr_graceful, cr->cr_graceful = 0;
+  cr->cr_offer_sent = cr->cr_answer_recv = 0;
+  cr->cr_offer_recv = cr->cr_answer_sent = 0;
 
-  msg = msg_copy(cr->cr_msg);
-  sip = sip_object(msg);
+  if (!ds->ds_leg && cr->cr_dialog) {
+    sip_t const *sip = cr->cr_sip;
+    ds->ds_leg = nta_leg_tcreate(nh->nh_nua->nua_nta,
+				 nua_stack_process_request, nh,
+				 SIPTAG_CALL_ID(sip->sip_call_id),
+				 SIPTAG_FROM(sip->sip_from),
+				 SIPTAG_TO(sip->sip_to),
+				 SIPTAG_CSEQ(sip->sip_cseq),
+				 TAG_END());
+  }
+
+  if (ds->ds_leg || !cr->cr_dialog) {
+    msg = msg_copy(cr->cr_msg);
+    sip = sip_object(msg);
+  }
 
   if (msg && sip) {
     cr->cr_restarting = 1;
@@ -2740,6 +3081,7 @@ int nua_base_client_response(nua_client_request_t *cr,
   nua_handle_t *nh = cr->cr_owner;
   sip_method_t method = cr->cr_method;
   nua_dialog_usage_t *du;
+  nua_client_request_t *cr_next;
 
   cr->cr_reporting = 1, nh->nh_ds->ds_reporting = 1;
 
@@ -2781,7 +3123,7 @@ int nua_base_client_response(nua_client_request_t *cr,
     if (cr->cr_terminated ||
 	(!du->du_ready && status >= 300 && nua_client_is_bound(cr))) {
       /* Usage has been destroyed */
-      nua_dialog_usage_remove(nh, nh->nh_ds, du), cr->cr_usage = NULL;
+      nua_dialog_usage_remove(nh, nh->nh_ds, du, cr, NULL), cr->cr_usage = NULL;
     }
     else if (cr->cr_graceful) {
       /* Terminate usage gracefully */
@@ -2794,16 +3136,18 @@ int nua_base_client_response(nua_client_request_t *cr,
       nua_dialog_remove(nh, nh->nh_ds, NULL), cr->cr_usage = NULL;
   }
 
+  cr->cr_phrase = NULL;
   cr->cr_reporting = 0, nh->nh_ds->ds_reporting = 0;
 
+  cr_next = nh->nh_ds->ds_cr;
+
   if (!nua_client_is_queued(cr) && !nua_client_is_bound(cr))
     nua_client_request_destroy(cr);
 
   if (method == sip_method_cancel)
     return 1;
 
-  return
-    nua_client_next_request(nh->nh_ds->ds_cr, method == sip_method_invite);
+  return nua_client_next_request(cr_next, method == sip_method_invite);
 }
 
 /** Send event, zap transaction but leave cr in list */
