@@ -75,7 +75,7 @@ SOFIA_BEGIN_DECLS
 #include "nua_params.h"
 #endif
 
-typedef struct event_s event_t;
+typedef struct event_s event_t, nua_signal_data_t;
 
 #define       NONE ((void *)-1)
 
@@ -109,7 +109,7 @@ typedef struct register_usage nua_registration_t;
 #define nua_handle_unref(nh) nua_handle_unref_by((nh), __func__)
 
 su_inline nua_handle_t *nua_handle_ref_by(nua_handle_t *nh,
-					      char const *by)
+					  char const *by)
 {
   if (nh)
     SU_DEBUG_0(("nua_handle_ref(%p) => "MOD_ZU" by %s\n", nh, 
@@ -120,11 +120,16 @@ su_inline nua_handle_t *nua_handle_ref_by(nua_handle_t *nh,
 
 su_inline int nua_handle_unref_by(nua_handle_t *nh, char const *by)
 {
-  if (nh)
-    SU_DEBUG_0(("nua_handle_unref(%p) => "MOD_ZU" by %s\n", nh, 
-		su_home_refcount((su_home_t *)nh) - 1,
-		by));
-  return su_home_unref((su_home_t *)nh);
+  if (nh) {
+    size_t refcount = su_home_refcount((su_home_t *)nh) - 1;
+    int freed =  su_home_unref((su_home_t *)nh);
+    if (freed) refcount = 0;
+    SU_DEBUG_0(("nua_handle_unref(%p) => "MOD_ZU" by %s\n",
+		nh, refcount, by));
+    return freed;
+  }
+
+  return 0; 
 }
 
 #endif
@@ -149,6 +154,7 @@ struct nua_handle_s
   nua_handle_t   *nh_identity;	/**< Identity */
 
   nua_handle_preferences_t *nh_prefs; /**< Preferences */
+#define nh_dprefs nh_nua->nua_dhandle->nh_prefs
 
   /* Handle type is determined by special event and flags. */
   nua_event_t     nh_special;	/**< Special event */
@@ -202,6 +208,8 @@ typedef struct nua_event_frame_s nua_event_frame_t;
 extern char const nua_internal_error[];
 
 #define NUA_INTERNAL_ERROR 900, nua_internal_error
+#define _NUA_INTERNAL_ERROR_AT(file, line) "Internal error at " file ":" #line
+#define NUA_ERROR_AT(file, line) 900, _NUA_INTERNAL_ERROR_AT(file, line)
 
 struct nua_s {
   su_home_t            nua_home[1];
@@ -211,28 +219,31 @@ struct nua_s {
   su_clone_r   	       nua_clone;
   su_task_r            nua_client;
 
-  su_network_changed_t *nua_nw_changed;
-
   nua_callback_f       nua_callback;
   nua_magic_t         *nua_magic;
 
   nua_event_frame_t   *nua_current;
   nua_saved_event_t    nua_signal[1];
 
+  /**< Used by stop-and-wait args calls */
+  tagi_t const        *nua_args;
+
   /* Engine state flags */
+  sip_time_t           nua_shutdown;
+
   unsigned             nua_shutdown_started:1; /**< Shutdown initiated */
   unsigned             nua_shutdown_final:1; /**< Shutdown is complete */
 
   unsigned             nua_from_is_set;
   unsigned :0;
   
-  /**< Used by stop-and-wait args calls */
-  tagi_t const        *nua_args;
-
   /**< Local SIP address. Contents are kept around for ever. */
-  sip_from_t          nua_from[1];
+  sip_from_t           nua_from[1];
+
+  /* ---------------------------------------------------------------------- */
 
   /* Protocol (server) side */
+  su_network_changed_t *nua_nw_changed;
 
   nua_registration_t *nua_registrations; /**< Active registrations */
 
@@ -244,21 +255,8 @@ struct nua_s {
   nta_agent_t        *nua_nta;
   su_timer_t         *nua_timer;
 
-  void         	      *nua_sip_parser;
-
-  sip_time_t           nua_shutdown;
-
-  /* Route */
-  sip_service_route_t *nua_service_route;
-
   /* User-agent parameters */
-  unsigned             nua_media_enable:1;
-
-  unsigned     	       :0;
-
-#if HAVE_SMIME		/* Start NRC Boston */
-  sm_object_t          *sm;
-#endif                  /* End NRC Boston */
+  nua_global_preferences_t nua_prefs[1];
 
   nua_handle_t        *nua_handles;
   nua_handle_t       **nua_handles_tail;
@@ -279,10 +277,19 @@ struct nua_s {
 #define __func__ "nua"
 #endif
 
+su_inline nua_t *nua_stack_ref(nua_t *nua)
+{ 
+  return (nua_t *)su_home_ref(nua->nua_home);
+}
+
+su_inline void nua_stack_unref(nua_t *nua)
+{ 
+  su_home_unref(nua->nua_home);
+}
+
 /* Internal prototypes */
 int  nua_stack_init(su_root_t *root, nua_t *nua);
 void nua_stack_deinit(su_root_t *root, nua_t *nua);
-void nua_stack_signal(nua_t *nua, su_msg_r msg, event_t *e);
 
 int nua_stack_init_transport(nua_t *nua, tagi_t const *tags);
 
@@ -307,6 +314,10 @@ typedef int nua_stack_signal_handler(nua_t *,
 				     nua_event_t, 
 				     tagi_t const *);
 
+void nua_move_signal(nua_saved_signal_t a[1], nua_saved_signal_t b[1]);
+nua_signal_data_t const *nua_signal_data(nua_saved_signal_t const saved[1]);
+void nua_destroy_signal(nua_saved_signal_t saved[1]);
+
 nua_stack_signal_handler 
   nua_stack_set_params, nua_stack_get_params,
   nua_stack_register, 
@@ -334,6 +345,8 @@ int nua_stack_event(nua_t *nua, nua_handle_t *nh, msg_t *msg,
 		    nua_event_t event, int status, char const *phrase,
 		    tagi_t const *tags);
 
+void nua_move_event(nua_saved_event_t a[1], nua_saved_event_t b[1]);
+
 nua_handle_t *nh_create_handle(nua_t *nua, nua_hmagic_t *hmagic, tagi_t *tags);
 
 nua_handle_t *nua_stack_incoming_handle(nua_t *nua,
@@ -369,6 +382,9 @@ sip_replaces_t *nua_stack_handle_make_replaces(nua_handle_t *handle,
 nua_handle_t *nua_stack_handle_by_replaces(nua_t *nua,
 					   sip_replaces_t const *r);
 
+nua_handle_t *nua_stack_handle_by_call_id(nua_t *nua, const char *call_id);
+
+
 /* ---------------------------------------------------------------------- */
 
 int nua_stack_set_defaults(nua_handle_t *nh, nua_handle_preferences_t *nhp);
@@ -416,8 +432,6 @@ extern tag_typedef_t _nutag_add_contact;
 
 /* ---------------------------------------------------------------------- */
 
-typedef unsigned longlong ull;
-
 #define SET_STATUS(_status, _phrase) status = _status, phrase = _phrase
 
 #define SET_STATUS2(_status, _phrase) status = _status, phrase = _phrase
@@ -433,11 +447,9 @@ typedef unsigned longlong ull;
 /* ---------------------------------------------------------------------- */
 /* Application side prototypes */
 
-void nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg, int always,
-		nua_event_t event, int status, char const *phrase,
-		tag_type_t tag, tag_value_t value, ...);
-
-void nua_event(nua_t *root_magic, su_msg_r sumsg, event_t *e);
+int nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg,
+	       nua_event_t event, int status, char const *phrase,
+	       tag_type_t tag, tag_value_t value, ...);
 
 SOFIA_END_DECLS
 
