@@ -41,9 +41,12 @@
 
 #include "sofia-sip/sip_parser.h"
 
+#include <stdio.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
+
 #include <assert.h>
 
 /* ====================================================================== */
@@ -172,7 +175,8 @@ static int sip_event_update(msg_common_t *h,
  * follows:
  * 
  * @code
- *    Allow-Events = ( "Allow-Events" | "u" ) ":" 1#event-type
+ *    Allow-Events = ( "Allow-Events" / "u" ) HCOLON event-type 
+ *                                           *(COMMA event-type)
  * @endcode
  *
  * The parsed Allow-Events header is stored in #sip_allow_events_t structure.
@@ -431,7 +435,7 @@ msg_hclass_t sip_publication_class[] =
 SIP_HEADER_CLASS(publication, "Publication", "", pub_params, single, 
 		 publication);
 
-static inline void sip_publication_update(sip_publication_t *pub);
+su_inline void sip_publication_update(sip_publication_t *pub);
 
 issize_t sip_publication_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
@@ -488,7 +492,7 @@ char *sip_publication_dup_one(sip_header_t *dst, sip_header_t const *src,
   return b;
 }
 
-static inline void sip_publication_update(sip_publication_t *pub)
+su_inline void sip_publication_update(sip_publication_t *pub)
 {
   size_t i;
 
