@@ -50,6 +50,7 @@
 #include <sofia-sip/hostdomain.h>
 
 
+#include <stdio.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -247,8 +248,8 @@ sip_contact_string_from_via(su_home_t *home,
     if (port || host_is_ip_address(host))
       transport = NULL;
   }
-  else if (port && host_is_ip_address(host) &&
-	   strcmp(port, SIP_DEFAULT_SERV) == 0) {
+  else if (port && strcmp(port, SIP_DEFAULT_SERV) == 0 &&
+	   (host_is_ip_address(host) || host_has_domain_invalid(host))) {
     port = NULL;
   }
 
@@ -258,10 +259,11 @@ sip_contact_string_from_via(su_home_t *home,
   /* Make transport parameter lowercase */
   if (transport && strlen(transport) < (sizeof _transport)) {
     char *s = strcpy(_transport, transport);
+    short c;
 
-    for (s = _transport; *s && *s != ';'; s++)
-      if (isupper(*s))
-	*s = tolower(*s);
+    for (s = _transport; (c = *s) && c != ';'; s++)
+      if (isupper(c))
+	*s = tolower(c);
 
     transport = _transport;
   }
@@ -362,17 +364,17 @@ sip_sanity_check(sip_t const *sip)
  */
 issize_t sip_header_field_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
-  assert(SIP_HDR_TEST(h));
-
   if (h && s && s[slen] == '\0') {
     size_t n = span_lws(s);
     s += n; slen -= n;
-    
+
     for (n = slen; n >= 1 && IS_LWS(s[n - 1]); n--)
       ;
     
     s[n] = '\0';
     
+    assert(SIP_HDR_TEST(h));
+
     return h->sh_class->hc_parse(home, h, s, slen);
   }
   else
@@ -410,7 +412,7 @@ char *sip_header_as_string(su_home_t *home, sip_header_t const *h)
   for (rv = su_alloc(home, len);
        rv;
        rv = su_realloc(home, rv, len)) {
-    ssize_t n = sip_header_field_e(s, sizeof(s), h, 0);
+    ssize_t n = sip_header_field_e(rv, len, h, 0);
     if (n > -1 && n + 1 <= len)
       break;
     if (n > -1)			/* glibc >2.1 */
@@ -881,8 +883,7 @@ sip_security_client_select(sip_security_client_t const *client,
  * decide whether to gracefully terminate or not, the
  * @a *return_graceful_terminate_usage is left unmodified.
  *
- * @sa 
- * http://www.ietf.org/internet-drafts/draft-ietf-sipping-dialogusage-02.txt
+ * @RFC5057
  */
 int sip_response_terminates_dialog(int response_code,
 				   sip_method_t method,
@@ -1058,8 +1059,6 @@ int sip_response_terminates_dialog(int response_code,
       usage in an existing dialog, no new usage is created and existing
       usages are unaffected.
     */
-    *return_graceful_terminate_usage = 0;
-    return 0;
 
   case 423: /** @par 423 Interval Too Brief 
 
@@ -1068,8 +1067,6 @@ int sip_response_terminates_dialog(int response_code,
       subscribe usage is not destroyed (or otherwise affected).  No
       other usages of the dialog are affected.
     */
-    *return_graceful_terminate_usage = 0;
-    return sip_method_subscribe == method ? terminate_usage : no_effect;
 
   case 428: /** @par 428 Use Identity Header
 
@@ -1077,8 +1074,6 @@ int sip_response_terminates_dialog(int response_code,
       the usage.  The usage is not affected.  The dialog is only
       affected by a change in its local @CSeq.  No other usages of the
       dialog are affected. */
-    *return_graceful_terminate_usage = 0;
-    return 0;
 
   case 429: /** @par 429 Provide Referrer Identity 
 
@@ -1120,7 +1115,7 @@ int sip_response_terminates_dialog(int response_code,
       This response indicates that the peer has lost its copy of the dialog
       state. The dialog and any usages sharing it are destroyed.
 
-The dialog
+      The dialog
       itself should not be destroyed unless this was the last usage.
       The effects of a 481 on a dialog and its usages are the most
       ambiguous of any final response.  There are implementations that
@@ -1420,3 +1415,4 @@ The dialog
 
   return 0;
 }
+
