@@ -42,21 +42,11 @@
 #include <sofia-sip/string0.h>
 #endif
 
-SOFIA_BEGIN_DECLS
+#ifndef MSG_HEADER_H
+#include <sofia-sip/msg_header.h>
+#endif
 
-/* @deprecated
- * These are just wrappers around msg_params_*() functions. 
- *
- * Use msg_header_*_param() functions instead.
- */
-SOFIAPUBFUN char const *sip_params_find(sip_param_t const pp[],
-					char const *token);
-SOFIAPUBFUN int sip_params_add(su_home_t *sh, 
-			       sip_param_t **pparams,
-			       char const *param);
-SOFIAPUBFUN int sip_params_cmp(sip_param_t const a[], sip_param_t const b[]);
-SOFIAPUBFUN int sip_params_replace(su_home_t *,
-				   sip_param_t **pparams, char const *param);
+SOFIA_BEGIN_DECLS
 
 SOFIAPUBFUN
 sip_contact_t *
@@ -148,7 +138,7 @@ union sip_pref
     usize_t spl_length;
   } sp_literal;
 
-  /** String (tag="&lt;foo&gt;"). */
+  /** String (tag="&lt;foo&gt;"). */ /* (tag="<foo>") */
   struct sp_string {
     enum sp_type sps_type;
     char const *sps_value;
@@ -159,8 +149,8 @@ union sip_pref
    */
   struct sp_range {
     enum sp_type spr_type;
-    double spr_lower;
-    double spr_upper;
+    double spr_lower;		/**< Lower limit. Lowest value is -DBL_MAX. */
+    double spr_upper;		/**< Upper limit. Highest value is DBL_MAX. */
   } sp_range;
 };
 
@@ -210,6 +200,13 @@ sip_security_client_t const *
 sip_security_client_select(sip_security_client_t const *client,
 			   sip_security_server_t const *server);
 
+/* Compatibility stuff */
+
+#define sip_params_add      	msg_params_add
+#define sip_params_cmp      	msg_params_cmp
+#define sip_params_replace  	msg_params_replace
+#define sip_params_find         msg_params_find
+
 SOFIA_END_DECLS
 
 #endif /** !defined(SIP_UTIL_H) */
