@@ -70,10 +70,6 @@ static char const __func__[] = "soa";
 /* ======================================================================== */
 
 /* Internal prototypes */
-void soa_set_activity(soa_session_t *ss,
-		      sdp_media_t const *,
-		      int remote);
-
 su_inline int soa_media_is_ready(soa_session_t const *ss);
 
 enum soa_sdp_kind { 
@@ -885,37 +881,41 @@ int soa_set_capability_sdp(soa_session_t *ss,
 
 /** Set capabilities */
 int 
-soa_base_set_capability_sdp(soa_session_t *ss, 
-			    sdp_session_t *sdp, char const *str0, isize_t len0)
+soa_base_set_capability_sdp(soa_session_t *ss,
+			    sdp_session_t *_sdp,
+			    char const *str0, isize_t len0)
 {
+  sdp_session_t sdp[1];
   sdp_origin_t o[1] = {{ sizeof(o) }};
   sdp_connection_t *c, c0[1] = {{ sizeof(c0) }};
   char c_address[64];
   sdp_time_t t[1] = {{ sizeof(t) }};
   sdp_media_t *m;
 
+  *sdp = *_sdp;
+
   if (sdp->sdp_origin)
     *o = *sdp->sdp_origin;
   else
     o->o_address = c0;
 
-  sdp->sdp_origin = o;
-
   if (soa_init_sdp_origin(ss, o, c_address) < 0)
     return -1;
 
-  if (!sdp->sdp_subject)
-    sdp->sdp_subject = "-";
+  sdp->sdp_origin = o;
 
-  sdp->sdp_time = t;
+  if (!sdp->sdp_subject)
+    sdp->sdp_subject = "-";	/* s=- */
+
+  sdp->sdp_time = t;		/* t=0 0 */
 
   /* Set port to zero - or should we check that port is already zero? */
   for (m = sdp->sdp_media; m; m = m->m_next)
     m->m_port = 0;
 
-  c = sdp->sdp_origin->o_address;
-
   if (sdp->sdp_connection == NULL) {
+    c = sdp->sdp_origin->o_address;
+
     for (m = sdp->sdp_media; m; m = m->m_next)
       if (m->m_connections == NULL)
 	break;
@@ -1171,7 +1171,7 @@ int soa_base_set_remote_sdp(soa_session_t *ss,
   if (!new_version)
     return 0;
     
-  soa_set_activity(ss, sdp->sdp_media, 1);
+  soa_set_activity(ss, sdp->sdp_media, soa_activity_remote);
 
   ss->ss_remote_version++;
   
@@ -1454,7 +1454,7 @@ int soa_base_generate_offer(soa_session_t *ss,
   if (!sdp)
     return -1;
 
-  soa_set_activity(ss, sdp->sdp_media, 0);
+  soa_set_activity(ss, sdp->sdp_media, soa_activity_local); /* Wanted activity */
 
   ss->ss_offer_sent = 1;
   ss->ss_answer_recv = 0;
@@ -1542,8 +1542,7 @@ int soa_base_generate_answer(soa_session_t *ss,
     su_free(ss->ss_home, ss->ss_rsession);
   ss->ss_rsession = rsession;
 
-  soa_set_activity(ss, l_sdp->sdp_media, 0);
-  soa_set_activity(ss, r_sdp->sdp_media, 1);
+  soa_set_activity(ss, l_sdp->sdp_media, soa_activity_session);
 
   ss->ss_offer_recv = 1;
   ss->ss_answer_sent = 1;
@@ -1624,8 +1623,7 @@ int soa_base_process_answer(soa_session_t *ss,
     su_free(ss->ss_home, ss->ss_rsession);
   ss->ss_rsession = rsession;
 
-  soa_set_activity(ss, l_sdp->sdp_media, 0);
-  soa_set_activity(ss, r_sdp->sdp_media, 1);
+  soa_set_activity(ss, l_sdp->sdp_media, soa_activity_session);
 
   ss->ss_answer_recv = 1;
   ss->ss_complete = 1;
@@ -1692,7 +1690,7 @@ int soa_base_process_reject(soa_session_t *ss,
   if (!l_sdp)
     return -1;
 
-  soa_set_activity(ss, l_sdp->sdp_media, 0);
+  soa_set_activity(ss, l_sdp->sdp_media, soa_activity_session);
 
   ss->ss_offer_sent = 0;
 
@@ -1783,8 +1781,7 @@ void soa_base_terminate(soa_session_t *ss, char const *option)
   ss->ss_oa_rounds = 0;
 
   soa_description_free(ss, ss->ss_remote);
-  soa_set_activity(ss, NULL, 0);
-  soa_set_activity(ss, NULL, 1);
+  soa_set_activity(ss, NULL, soa_activity_session);
 }
 
 /** Return true if the SDP Offer/Answer negotation is complete.
@@ -1889,56 +1886,75 @@ int soa_media_is_ready(soa_session_t const *ss)
 
 void soa_set_activity(soa_session_t *ss,
 		      sdp_media_t const *m,
-		      int remote)
+		      enum soa_activity activity)
 {
   struct soa_media_activity *ma;
   sdp_connection_t const *c;
-  int mode;
-  int ma_audio = SOA_ACTIVE_DISABLED;
-  int ma_video = SOA_ACTIVE_DISABLED;
-  int ma_chat = SOA_ACTIVE_DISABLED;
-  int ma_image = SOA_ACTIVE_DISABLED;
-  int *p;
+  int mode, swap;
+  int l_audio = SOA_ACTIVE_DISABLED, r_audio = SOA_ACTIVE_DISABLED;
+  int l_video = SOA_ACTIVE_DISABLED, r_video = SOA_ACTIVE_DISABLED;
+  int l_chat = SOA_ACTIVE_DISABLED,  r_chat = SOA_ACTIVE_DISABLED;
+  int l_image = SOA_ACTIVE_DISABLED, r_image = SOA_ACTIVE_DISABLED;
 
-  remote = !!remote;
-
-  ma = remote ? ss->ss_remote_activity : ss->ss_local_activity;
+  int *l, *r;
 
   for (; m; m = m->m_next) {
     if (m->m_type == sdp_media_audio)
-      p = &ma_audio;
+      l = &l_audio, r = &r_audio;
     else if (m->m_type == sdp_media_video)
-      p = &ma_video;
+      l = &l_video, r = &r_video;
     else if (m->m_type == sdp_media_image)
-      p = &ma_image;
+      l = &l_image, r = &r_image;
     else if (strcasecmp(m->m_type_name, "message") == 0)
-      p = &ma_chat;
+      l = &l_chat, r = &r_chat;
     else
       continue;
 
     if (m->m_rejected) {
-      if (*p < 0)
-	*p = SOA_ACTIVE_REJECTED;
+      if (*l < 0) *l = SOA_ACTIVE_REJECTED;
+      if (*r < 0) *r = SOA_ACTIVE_REJECTED;
       continue;
     }
 
-    mode = m->m_mode;
+    mode = m->m_mode, swap = ((mode << 1) & 2) | ((mode >> 1) & 1);
 
     c = sdp_media_connections((sdp_media_t *)m);
 
-    if (remote != (c && c->c_mcast))
-      mode = ((mode << 1) & 2) | ((mode >> 1) & 1);
-
-    if (*p < 0)
-      *p = mode;
-    else
-      *p |= mode;
+    switch (activity) {
+    case soa_activity_local:
+      *l &= SOA_ACTIVE_SENDRECV;
+      *l |= c && c->c_mcast ? swap : mode;
+      break;
+    case soa_activity_remote:
+      *r &= SOA_ACTIVE_SENDRECV;
+      *r = c && c->c_mcast ? mode : swap;
+      break;
+    case soa_activity_session:
+      *l &= SOA_ACTIVE_SENDRECV;
+      *l |= c && c->c_mcast ? swap : mode;
+      *r &= SOA_ACTIVE_SENDRECV;
+      *r = c && c->c_mcast ? swap : mode;
+      break;
+    }
   }
 
-  ma->ma_audio = ma_audio;
-  ma->ma_video = ma_video;
-  ma->ma_image = ma_image;
-  ma->ma_chat = ma_chat;
+  if (activity == soa_activity_local ||
+      activity == soa_activity_session) {
+    ma = ss->ss_local_activity;
+    ma->ma_audio = l_audio;
+    ma->ma_video = l_video;
+    ma->ma_image = l_image;
+    ma->ma_chat = l_chat;
+  }
+
+  if (activity == soa_activity_remote ||
+      activity == soa_activity_session) {
+    ma = ss->ss_remote_activity;
+    ma->ma_audio = r_audio;
+    ma->ma_video = r_video;
+    ma->ma_image = r_image;
+    ma->ma_chat = r_chat;
+  }
 }
 
 /* ----------------------------------------------------------------------*/
@@ -1965,6 +1981,9 @@ int soa_set_sdp(soa_session_t *ss,
   sdp_parser_t *parser = NULL;
   sdp_session_t sdp[1];
 
+  if (ss == NULL)
+    return -1;
+
   switch (what) {
   case soa_capability_sdp_kind:
     ssd = ss->ss_caps;
@@ -1982,15 +2001,16 @@ int soa_set_sdp(soa_session_t *ss,
     return -1;
   }
 
-  if (sdp_str && str_len == -1)
-    str_len = strlen(sdp_str);
-
-  if (sdp0)
+  if (sdp0) {
     new_version = sdp_session_cmp(sdp0, ssd->ssd_sdp) != 0;
-  else if (sdp_str)
+    sdp_str = NULL, str_len = -1;
+  }
+  else if (sdp_str) {
+    if (str_len == -1)
+      str_len = strlen(sdp_str);
     new_version = !ssd->ssd_unparsed ||
-      str0ncmp(sdp_str, ssd->ssd_unparsed, str_len) != 0 ||
-      ssd->ssd_unparsed[str_len] != '\0';
+      str0ncmp(sdp_str, ssd->ssd_unparsed, str_len + 1) != 0;
+  }
   else
     return (void)su_seterrno(EINVAL), -1;
 
