@@ -731,10 +731,11 @@ int soa_sdp_upgrade(soa_session_t *ss,
   o_media = su_zalloc(home, (Ns + 1) * (sizeof *o_media));
   u_media = su_zalloc(home, (Nu + 1) * (sizeof *u_media));
   r_media = su_zalloc(home, (Nr + 1) * (sizeof *r_media));
+  if (!s_media || !o_media || !u_media || !r_media)
+    return -1;
 
   um = sdp_media_dup_all(home, user->sdp_media, session); 
-
-  if (!s_media || !u_media || !r_media || !um)
+  if (!um && user->sdp_media)
     return -1;
 
   u2s = su_alloc(home, (Nu + 1) * sizeof(*u2s));
@@ -805,33 +806,37 @@ int soa_sdp_upgrade(soa_session_t *ss,
 	soa_sdp_media_upgrade_rtpmaps(ss, m, rm);
     }
   }
-  else if (sss->sss_ordered_user) {
-    /* Update session with unused media in u_media */
-
-    if (!sss->sss_reuse_rejected) {
-      /* Mark previously used slots */
-      for (i = 0; i < Ns; i++) {
-	if (s_media[i])
-	  continue;
-	s_media[i] = soa_sdp_make_rejected_media(home, o_media[i], session, 0);
-      }
-    }
-
-    for (j = 0; j < Nu; j++) {
-      if (u_media[j] == SDP_MEDIA_NONE)
-	continue;
-
-      for (i = 0; i < size - 1; i++) {
-	if (s_media[i] != NULL)
-	  continue;
-	s_media[i] = u_media[j], u_media[j] = SDP_MEDIA_NONE;
-	u2s[j] = i, s2u[i] = j;
-      }
-
-      assert(i != size);
-    }
-  }
   else {
+
+    if (sss->sss_ordered_user) {
+      /* Update session with unused media in u_media */
+
+      if (!sss->sss_reuse_rejected) {
+	/* Mark previously used slots */
+	for (i = 0; i < Ns; i++) {
+	  if (s_media[i])
+	    continue;
+	  s_media[i] = 
+	    soa_sdp_make_rejected_media(home, o_media[i], session, 0);
+	}
+      }
+
+      for (j = 0; j < Nu; j++) {
+	if (u_media[j] == SDP_MEDIA_NONE)
+	  continue;
+
+	for (i = 0; i < size - 1; i++) {
+	  if (s_media[i] == NULL) {
+	    s_media[i] = u_media[j], u_media[j] = SDP_MEDIA_NONE;
+	    u2s[j] = i, s2u[i] = j;
+	    break;
+	  }
+	}
+
+	assert(i != size - 1);
+      }
+    }
+
     /* Match unused user media by media types with the existing session */
     for (i = 0; i < Ns; i++) {
       if (s_media[i])
@@ -864,11 +869,6 @@ int soa_sdp_upgrade(soa_session_t *ss,
   }
   *mm = NULL;
 
-#ifndef NDEBUG
-  for (j = i; j < size; j++)
-    assert(s2u[j] == U2S_NOT_USED);
-#endif
-
   s2u[size = i] = U2S_SENTINEL;
   *return_u2s = u2s;
   *return_s2u = s2u;
@@ -974,58 +974,28 @@ int soa_sdp_reject(su_home_t *home,
   return 0;
 }
 
-/** Check if @a session mode should be changed. */ 
+
+/** Update mode within session.
+ *
+ * @sa soatag_hold
+ *
+ * @retval 1 if session was changed (or to be changed, if @a dryrun is nonzero)
+ */ 
 static
-int soa_sdp_mode_set_is_needed(sdp_session_t const *session,
-			       sdp_session_t const *remote,
-			       char const *hold)
-{
-  sdp_media_t const *sm, *rm, *rm_next;
-  int hold_all;
-  sdp_mode_t recv_mode;
-
-  SU_DEBUG_7(("soa_sdp_mode_set_is_needed(%p, %p, \"%s\"): called\n",
-	      (void *)session, (void *)remote, hold ? hold : ""));
-
-  if (!session )
-    return 0;
-
-  hold_all = str0cmp(hold, "*") == 0;
-
-  rm = remote ? remote->sdp_media : NULL, rm_next = NULL;
-
-  for (sm = session->sdp_media; sm; sm = sm->m_next, rm = rm_next) {
-    rm_next = rm ? rm->m_next : NULL;
-
-    if (sm->m_rejected)
-      continue;
-
-    if (rm) {
-      /* Mode bits do not match */
-      if (((rm->m_mode & sdp_recvonly) == sdp_recvonly)
-	  != ((sm->m_mode & sdp_sendonly) == sdp_sendonly))
-	return 1;
-    }
-
-    recv_mode = sm->m_mode & sdp_recvonly;
-    if (recv_mode && hold &&
-	(hold_all || strcasestr(hold, sm->m_type_name)))
-      return 1;
-  }
-
-  return 0;
-}
-
-
-/** Update mode within session */ 
-static
-int soa_sdp_mode_set(sdp_session_t *session,
+int soa_sdp_mode_set(sdp_session_t const *user,
+		     int const *s2u,
+		     sdp_session_t *session,
 		     sdp_session_t const *remote,
-		     char const *hold)
+		     char const *hold,
+		     int dryrun)
 {
   sdp_media_t *sm;
-  sdp_media_t const *rm, *rm_next;
+  sdp_media_t const *rm, *rm_next, *um;
+  int retval = 0, i, j;
   int hold_all;
+  int inactive_all;
+  int inactive = 0;
+  char const *hold_media = NULL;
   sdp_mode_t send_mode, recv_mode;
 
   SU_DEBUG_7(("soa_sdp_mode_set(%p, %p, \"%s\"): called\n",
@@ -1037,25 +1007,68 @@ int soa_sdp_mode_set(sdp_session_t *session,
   rm = remote ? remote->sdp_media : NULL, rm_next = NULL;
 
   hold_all = str0cmp(hold, "*") == 0;
+  inactive_all = str0cmp(hold, "#") == 0;
 
-  for (sm = session->sdp_media; sm; sm = sm->m_next, rm = rm_next) {
+  i = 0;
+
+  for (sm = session->sdp_media; sm; sm = sm->m_next, rm = rm_next, i++) {
     rm_next = rm ? rm->m_next : NULL;
+    inactive = 0;
 
     if (sm->m_rejected)
       continue;
 
-    send_mode = sdp_sendonly;
+    assert(s2u);
+
+    for (j = 0, um = user->sdp_media; j != s2u[i]; um = um->m_next, j++)
+      assert(um);
+    if (um == NULL) {
+      if (dryrun)
+	return 1;
+      else
+	retval = 1;
+      sm->m_rejected = 1;
+      sm->m_mode = sdp_inactive;
+      sm->m_port = 0;
+      continue;
+    }
+
+    send_mode = um->m_mode & sdp_sendonly;
     if (rm)
       send_mode = (rm->m_mode & sdp_recvonly) ? sdp_sendonly : 0;
 
-    recv_mode = sm->m_mode & sdp_recvonly;
-    if (recv_mode && hold && (hold_all || strcasestr(hold, sm->m_type_name)))
-      recv_mode = 0;
+    recv_mode = um->m_mode & sdp_recvonly;
 
-    sm->m_mode = recv_mode | send_mode;
+    if (rm && rm->m_mode == sdp_inactive) {
+      send_mode = recv_mode = 0;
+    }
+    else if (inactive_all) {
+      send_mode = recv_mode = 0;
+    }
+    else if (hold_all) {
+      recv_mode = 0;
+    }
+    else if (hold && (hold_media = strcasestr(hold, sm->m_type_name))) {
+      recv_mode = 0;
+      hold_media += strlen(sm->m_type_name);
+      hold_media += strspn(hold_media, " \t");
+      if (hold_media[0] == '=') {
+	hold_media += strspn(hold, " \t");
+	if (strncasecmp(hold_media, "inactive", strlen("inactive")) == 0)
+	  recv_mode = send_mode = 0;
+      }
+    }
+
+    if (sm->m_mode != (unsigned)(recv_mode | send_mode)) {
+      if (dryrun)
+	return 1;
+      else
+	retval = 1;
+      sm->m_mode = recv_mode | send_mode;
+    }
   }
 
-  return 0;
+  return retval;
 }
 
 enum offer_answer_action {
@@ -1104,6 +1117,8 @@ static int offer_answer_step(soa_session_t *ss,
 
   if (action == generate_offer)
     remote = NULL;
+  else if (remote == NULL)
+    return soa_set_status(ss, 500, "No remote SDP");
 
   /* Pre-negotiation Step: Expand truncated remote SDP */
   if (local && remote) switch (action) {
@@ -1114,6 +1129,8 @@ static int offer_answer_step(soa_session_t *ss,
       SU_DEBUG_5(("%s: remote %s is truncated: expanding\n",
 		  by, action == generate_answer ? "offer" : "answer"));
       remote = soa_sdp_expand_media(tmphome, remote, local);
+      if (remote == NULL)
+	return soa_set_status(ss, 500, "Cannot expand remote session");
     }
   default:
     break;
@@ -1159,7 +1176,8 @@ static int offer_answer_step(soa_session_t *ss,
       *local0 = *local, local = local0;
     SU_DEBUG_7(("soa_static(%p, %s): %s\n", (void *)ss, by, 
 		"upgrade with local description"));
-    soa_sdp_upgrade(ss, tmphome, local, user, NULL, &u2s, &s2u);
+    if (soa_sdp_upgrade(ss, tmphome, local, user, NULL, &u2s, &s2u) < 0)
+      goto internal_error;
     break;
   case generate_answer:
     /* Upgrade local SDP based on remote SDP */
@@ -1171,7 +1189,8 @@ static int offer_answer_step(soa_session_t *ss,
 	*local0 = *local, local = local0;
       SU_DEBUG_7(("soa_static(%p, %s): %s\n", (void *)ss, by,
 		  "upgrade with remote description"));
-      soa_sdp_upgrade(ss, tmphome, local, user, remote, &u2s, &s2u);
+      if (soa_sdp_upgrade(ss, tmphome, local, user, remote, &u2s, &s2u) < 0)
+	goto internal_error;
     }
     break;
   case process_answer:
@@ -1213,16 +1232,22 @@ static int offer_answer_step(soa_session_t *ss,
 
   /* Step D: Set media mode bits */
   switch (action) {
+    int const *s2u_;
+
   case generate_offer:
   case generate_answer:
   case process_answer:
-    if (soa_sdp_mode_set_is_needed(local, remote, ss->ss_hold)) {
+    s2u_ = s2u;
+
+    if (!s2u_) s2u_ = sss->sss_s2u;
+
+    if (soa_sdp_mode_set(user, s2u_, local, remote, ss->ss_hold, 1)) {
       if (local != local0) {
 	*local0 = *local, local = local0;
 	DUP_LOCAL(local);
       }
 
-      soa_sdp_mode_set(local, remote, ss->ss_hold);
+      soa_sdp_mode_set(user, s2u_, local, remote, ss->ss_hold, 0);
     }
     break;
   default:
