@@ -163,10 +163,10 @@ struct soa_session
 
   struct soa_media_activity
   {
-    int ma_audio:4; /**< Audio activity (send/recv) */
-    int ma_video:4; /**< Video activity (send/recv) */
-    int ma_image:4; /**< Image activity (send/recv) for JPIP */
-    int ma_chat:4;  /**< Chat activity (send/recv) */
+    unsigned ma_audio:4; /**< Audio activity (send/recv) */
+    unsigned ma_video:4; /**< Video activity (send/recv) */
+    unsigned ma_image:4; /**< Image activity (send/recv) for JPIP */
+    unsigned ma_chat:4;  /**< Chat activity (send/recv) */
   } ss_local_activity[1], ss_remote_activity[1];
 
   /** Capabilities as specified by application */
@@ -238,8 +238,11 @@ SOFIAPUBFUN int soa_has_received_sdp(soa_session_t const *ss);
 SOFIAPUBFUN int soa_set_status(soa_session_t *ss,
 			       int status, char const *phrase);
 
+enum soa_activity { soa_activity_local, soa_activity_remote, soa_activity_session };
+
 SOFIAPUBFUN void soa_set_activity(soa_session_t *ss, 
-				  sdp_media_t const *m, int remote);
+				  sdp_media_t const *m, 
+				  enum soa_activity activity);
 
 SOFIAPUBFUN int soa_description_set(soa_session_t *ss, 
 				    struct soa_description *ssd,
