@@ -22,7 +22,7 @@
  *
  */
 
-/**
+/**@internal
  * @file stun.c STUN client module
  *
  * See RFC 3489/3489bis for further information.
@@ -39,12 +39,14 @@
 
 #include <assert.h>
 #include <string.h>
+#include <time.h>
+#include <stdlib.h>
 
 #define SU_ROOT_MAGIC_T struct stun_magic_t
 
-#include "sofia-sip/stun.h"
+#include <sofia-sip/stun.h>
 #include "stun_internal.h"
-#include "sofia-sip/stun_tag.h"
+#include <sofia-sip/stun_tag.h>
 
 #include <sofia-sip/su_alloc.h>
 #include <sofia-sip/su_tagarg.h>
@@ -52,19 +54,14 @@
 #include <sofia-sip/su.h>
 #include <sofia-sip/su_localinfo.h>
 
-#if HAVE_WINSOCK2_H
-#include <winsock2.h>
-#include <ws2tcpip.h>
+#if HAVE_NETINET_TCP_H
+#include <netinet/tcp.h>
 #endif
 
 #if HAVE_OPENSSL
 #include <openssl/opensslv.h>
 #endif
 
-#if !defined(ETIMEDOUT) && defined(_WIN32)
-#define ETIMEDOUT WSAETIMEDOUT
-#endif
-
 /* Missing socket symbols */
 #ifndef SOL_TCP
 #define SOL_TCP IPPROTO_TCP
@@ -77,10 +74,14 @@
 static char const __func__[] = "stun";
 #endif
 
-/** STUN log. */
-su_log_t stun_log[] = { SU_LOG_INIT("stun", "STUN_DEBUG", 3) }; 
+#ifndef SU_DEBUG
+#define SU_DEBUG 3
+#endif
 
-/**@var STUN_DEBUG
+/** STUN log. */
+su_log_t stun_log[] = { SU_LOG_INIT("stun", "STUN_DEBUG", SU_DEBUG) }; 
+
+/**@var char const STUN_DEBUG[]
  *
  * Environment variable determining the debug log level for @b stun module.
  *
@@ -89,7 +90,7 @@ su_log_t stun_log[] = { SU_LOG_INIT("stun", "STUN_DEBUG", 3) };
  * 
  * @sa <sofia-sip/su_debug.h>, stun_log, SOFIA_DEBUG
  */
-extern char const STUN__DEBUG[];
+extern char const STUN_DEBUG[];
 
 enum {
   STUN_SENDTO_TIMEOUT = 1000,
@@ -806,7 +807,7 @@ int assign_socket(stun_discovery_t *sd, su_socket_t s, int register_socket)
 
     if ((err = bind(s, &su->su_sa, sulen)) < 0) {
       SU_DEBUG_3(("%s: bind(%s:%u): %s\n",  __func__, 
-		  inet_ntop(su->su_family, SU_ADDR(su), addr, sizeof(addr)),
+		  su_inet_ntop(su->su_family, SU_ADDR(su), addr, sizeof(addr)),
 		  (unsigned) ntohs(su->su_port),
 		  su_strerror(su_errno())));
       return -1;
@@ -820,7 +821,7 @@ int assign_socket(stun_discovery_t *sd, su_socket_t s, int register_socket)
   memcpy(&sd->sd_bind_addr, su, sulen);
 
   SU_DEBUG_3(("%s: local socket is bound to %s:%u\n", __func__,
-	      inet_ntop(su->su_family, SU_ADDR(su), addr, sizeof(addr)),
+	      su_inet_ntop(su->su_family, SU_ADDR(su), addr, sizeof(addr)),
 	      (unsigned) ntohs(su->su_port)));
 
   events = SU_WAIT_IN | SU_WAIT_ERR;
@@ -867,7 +868,7 @@ static int get_localinfo(int family, su_sockaddr_t *su, socklen_t *return_len)
       memcpy(su, li->li_addr, *return_len = li->li_addrlen);
 
       SU_DEBUG_3(("%s: using local address %s\n", __func__, 
-		  inet_ntop(family, SU_ADDR(su), addr, sizeof(addr))));
+		  su_inet_ntop(family, SU_ADDR(su), addr, sizeof(addr))));
       break;
     }
 
@@ -1696,7 +1697,7 @@ static int stun_bind_callback(stun_magic_t *m, su_wait_t *w, su_wakeup_arg_t *ar
 
   
   SU_DEBUG_5(("%s: response from server %s:%u\n", __func__,
-	      inet_ntop(recv.su_family, SU_ADDR(&recv), addr, sizeof(addr)),
+	      su_inet_ntop(recv.su_family, SU_ADDR(&recv), addr, sizeof(addr)),
 	      ntohs(recv.su_port)));
 
   debug_print(&binding_response.enc_buf);      
@@ -1894,23 +1895,23 @@ static int process_test_lifetime(stun_request_t *req, stun_msg_t *binding_respon
   if ((req->sr_state == stun_req_timeout) && (req->sr_from_y == -1)) {
     SU_DEBUG_0(("%s: lifetime determination failed.\n", __func__));
     sd->sd_state = stun_discovery_timeout;
+    req->sr_state = stun_req_dispose_me;
 
     /* Use per discovery specific callback */
     if (sd->sd_callback)
       sd->sd_callback(sd->sd_magic, sh, sd, action, sd->sd_state);
 
-    req->sr_state = stun_req_dispose_me;
     return 0;
   }
 
   if (abs(sd->sd_lt_cur - sd->sd_lt) <= STUN_LIFETIME_CI) {
     sd->sd_state = stun_discovery_done;
+    req->sr_state = stun_req_dispose_me;
 
     /* Use per discovery specific callback */
     if (sd->sd_callback)
       sd->sd_callback(sd->sd_magic, sh, sd, action, sd->sd_state);
 
-    req->sr_state = stun_req_dispose_me;
     return 0;
   }
 
@@ -1997,12 +1998,11 @@ static int action_bind(stun_request_t *req, stun_msg_t *binding_response)
   memcpy(sd->sd_addr_seen_outside, sa, sizeof(su_sockaddr_t));
 
   sd->sd_state = stun_discovery_done;
+  req->sr_state = stun_req_dispose_me;
   
   if (sd->sd_callback)
     sd->sd_callback(sd->sd_magic, sh, sd, action, sd->sd_state);
 
-  req->sr_state = stun_req_dispose_me;
-
   return 0;
 }
 
@@ -2017,7 +2017,7 @@ static int priv_find_matching_localadress(su_sockaddr_t *su)
   char addr[SU_ADDRSIZE];
 
   SU_DEBUG_5(("%s: checking if %s is a local address.\n", __func__, 
-	      inet_ntop(AF_INET, SU_ADDR(su), addr, sizeof(addr))));
+	      su_inet_ntop(AF_INET, SU_ADDR(su), addr, sizeof(addr))));
 
   hints->li_family = af = su->su_family;
 
@@ -2035,7 +2035,7 @@ static int priv_find_matching_localadress(su_sockaddr_t *su)
     }
 
     SU_DEBUG_9(("%s: skipping local address %s.\n", __func__, 
-		inet_ntop(af, SU_ADDR(li->li_addr), addr, sizeof(addr))));
+		su_inet_ntop(af, SU_ADDR(li->li_addr), addr, sizeof(addr))));
   }  
 
   su_freelocalinfo(res);
@@ -2052,9 +2052,9 @@ static void priv_mark_discovery_done(stun_discovery_t *sd,
 				     stun_request_t *req)
 {
   sd->sd_state = stun_discovery_done;
+  req->sr_state = stun_req_dispose_me;
   if (sd->sd_callback)
     sd->sd_callback(sd->sd_magic, sh, sd, action, sd->sd_state);
-  req->sr_state = stun_req_dispose_me;
 }
 
 /**
@@ -2289,12 +2289,12 @@ static void stun_sendto_timer_cb(su_root_magic_t *magic,
     switch (action) {
     case stun_action_binding_request:
       sd->sd_state = stun_discovery_timeout;
+      req->sr_state = stun_req_dispose_me;
 
       /* Use per discovery specific callback */
       if (sd->sd_callback)
 	sd->sd_callback(sd->sd_magic, sh, sd, action, sd->sd_state);
 
-      req->sr_state = stun_req_dispose_me;
       break;
 
     case stun_action_test_nattype:
@@ -2761,7 +2761,7 @@ int stun_test_lifetime(stun_handle_t *sh,
   }
 
   SU_DEBUG_3(("%s: socket y bound to %s:%u\n", __func__,
-	      inet_ntop(y_addr.su_family, SU_ADDR(&y_addr), addr, sizeof(addr)),
+	      su_inet_ntop(y_addr.su_family, SU_ADDR(&y_addr), addr, sizeof(addr)),
 	      (unsigned) ntohs(y_addr.su_port)));
 
   req->sr_from_y = -1;
@@ -2949,7 +2949,7 @@ int stun_keepalive(stun_handle_t *sh,
   req = stun_request_create(sd);
 
   SU_DEBUG_3(("%s: Starting to send STUN keepalives to %s:%u\n", __func__, 
-	      inet_ntop(sa->su_family, SU_ADDR(sa), addr, sizeof(addr)),
+	      su_inet_ntop(sa->su_family, SU_ADDR(sa), addr, sizeof(addr)),
 	      (unsigned) ntohs(sa->su_port)));
   
   if (stun_make_binding_req(sh, req, req->sr_msg, 0, 0) < 0 ||
