@@ -22,7 +22,7 @@
  *
  */
 
-/**
+/**@internal
  * @file stun_mini.c
  * @brief Minimal stun server 
  * 
@@ -39,6 +39,8 @@
 #include "stun_internal.h"
 
 #include <assert.h>
+#include <string.h>
+#include <stdlib.h>
 
 typedef struct stun_bound_s stun_bound_t;
 
@@ -182,7 +184,7 @@ void stun_mini_request(stun_mini_t *mini,
     struct sockaddr_in const *sin = from;
 
     if (sin->sin_family == AF_INET)
-      inet_ntop(sin->sin_family, &sin->sin_addr, buffer, sizeof buffer);
+      su_inet_ntop(sin->sin_family, &sin->sin_addr, buffer, sizeof buffer);
     else
       sprintf(buffer, "<af=%u>", (unsigned)sin->sin_family);
 
