@@ -22,7 +22,7 @@
  *
  */
 
-/**@CFILE stun_tag.c  Tags and tag lists for Offer/Answer Engine
+/**@internal @CFILE stun_tag.c  Tags and tag lists for Offer/Answer Engine
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>
  * @author Martti Mela <Martti.Mela@nokia.com>
@@ -36,6 +36,7 @@
 #define TAG_NAMESPACE "stun"
 
 #include <sofia-sip/su_tag_class.h>
+#include <sofia-sip/stun_tag.h>
 
 /**@def STUNTAG_ANY()
  * 
@@ -159,7 +160,7 @@ tag_typedef_t stuntag_register_events = BOOLTAG_TYPEDEF(register_events);
  *    int (stun_action_t)
  *
  * @par Values
- *    See types for stun_action_t in stun.h
+ *    See types for stun_action_t in <sofia-sip/stun.h>
  *
  * Corresponding tag taking reference parameter is STUNTAG_ACTION_REF()
  */
