@@ -43,7 +43,7 @@ SOFIA_BEGIN_DECLS
 /**@page tagarg Tagarg Functions
  *
  * A @em tagarg function may be called with a varying number of tagged
- * arguments.  The include file <su_tagarg.h> declares a type ta_list and
+ * arguments.  The include file <sofia-sip/su_tagarg.h> declares a type ta_list and
  * defines four macros (ta_start(), ta_args(), ta_tags() and ta_end()) for
  * accessing the argument list.
  *
@@ -66,10 +66,11 @@ SOFIA_BEGIN_DECLS
  * TAG_NULL() or TAG_NEXT().
  */
 
-/**Object for accessing tagged argument lists.
+/**Structure for accessing tagged argument lists.
  *
- * The called function must declare an object of type ta_list which is
- * used by the macros ta_start(), ta_args(), ta_tags(), and ta_end().
+ * The function called with tagged arguments must declare an object of type
+ * ta_list which is used by the macros ta_start(), ta_args(), ta_tags(), and
+ * ta_end().
  *
  * If a tagged list is not finished with TAG_END(), TAG_NULL(), or
  * TAG_NEXT() items, random errors may occur.
@@ -141,8 +142,11 @@ typedef struct {
     if (ta_start__tag != NULL &&					\
 	ta_start__tag != tag_null &&					\
 	ta_start__tag != tag_next) {					\
+      va_list ta_start__ap;						\
+      va_copy(ta_start__ap, (ta).ap);					\
       (ta).tl[1].t_tag = tag_next;					\
-      (ta).tl[1].t_value = (tag_value_t)tl_vlist((ta).ap);		\
+      (ta).tl[1].t_value = (tag_value_t)tl_vlist(ta_start__ap);		\
+      va_end(ta_start__ap);						\
     } else {								\
       (ta).tl[1].t_value = 0; (ta).tl[1].t_value = (tag_value_t)0;	\
     }									\
