@@ -186,6 +186,7 @@
 #include "sofia-sip/su_alloc_stat.h"
 #include "sofia-sip/su_errno.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
 #include <memory.h>
@@ -193,17 +194,24 @@
 
 #include <assert.h>
 
-void (*su_home_locker)(void *mutex);
-void (*su_home_unlocker)(void *mutex);
+int (*_su_home_locker)(void *mutex);
+int (*_su_home_unlocker)(void *mutex);
 
-void (*su_home_mutex_locker)(void *mutex);
-void (*su_home_mutex_unlocker)(void *mutex);
+int (*_su_home_mutex_locker)(void *mutex);
+int (*_su_home_mutex_trylocker)(void *mutex);
+int (*_su_home_mutex_unlocker)(void *mutex);
 
-void (*su_home_destroy_mutexes)(void *mutex);
+void (*_su_home_destroy_mutexes)(void *mutex);
+
+#if HAVE_FREE_NULL
+#define safefree(x) free((x))
+#else
+su_inline void safefree(void *b) { b ? free(b) : (void)0; }
+#endif
 
 #define MEMLOCK(h)   \
-  (((h) && (h)->suh_lock ? su_home_locker((h)->suh_lock) : (void)0), (h)->suh_blocks)
-#define UNLOCK(h) (((h) && (h)->suh_lock ? su_home_unlocker((h)->suh_lock) : (void)0), NULL)
+  ((void)((h) && (h)->suh_lock ? _su_home_locker((h)->suh_lock) : 0), (h)->suh_blocks)
+#define UNLOCK(h) ((void)((h) && (h)->suh_lock ? _su_home_unlocker((h)->suh_lock) : 0), NULL)
 
 #ifdef NDEBUG
 #define MEMCHECK 0
@@ -275,7 +283,7 @@ size_t su_block_find_collision, su_block_find_collision_used,
   su_block_find_collision_size;
 #endif
 
-su_inline su_alloc_t *su_block_find(su_block_t *b, void const *p)
+su_inline su_alloc_t *su_block_find(su_block_t const *b, void const *p)
 {
   size_t h, h0, probe;
 
@@ -298,8 +306,10 @@ su_inline su_alloc_t *su_block_find(su_block_t *b, void const *p)
   probe = (b->sub_n > SUB_P) ? SUB_P : 1;
 
   do {
-    if (b->sub_nodes[h].sua_data == p)
-      return &b->sub_nodes[h];
+    if (b->sub_nodes[h].sua_data == p) {
+      su_alloc_t const *retval = &b->sub_nodes[h];
+      return (su_alloc_t *)retval; /* discard const */
+    }
     h += probe;
     if (h >= b->sub_n)
       h -= b->sub_n;
@@ -498,7 +508,7 @@ void *sub_alloc(su_home_t *home,
 
       subhome->suh_blocks = su_hash_alloc(SUB_N);
       if (!subhome->suh_blocks) 
-	return (void)free(data), NULL;
+	return (void)safefree(data), NULL;
 
       subhome->suh_size = (unsigned)size;
       subhome->suh_blocks->sub_parent = home;
@@ -553,7 +563,7 @@ void *su_home_new(isize_t size)
     if (home->suh_blocks)
       home->suh_blocks->sub_hauto = 0;
     else
-      free(home), home = NULL;
+      safefree(home), home = NULL;
   }
 
   return home;
@@ -668,7 +678,7 @@ int su_home_unref(su_home_t *home)
     int hauto = sub->sub_hauto;
     _su_home_deinit(home);
     if (!hauto)
-      free(home);
+      safefree(home);
     /* UNLOCK(home); */
     return 1;
   }
@@ -778,7 +788,10 @@ void *su_alloc(su_home_t *home, isize_t size)
  */
 void su_free(su_home_t *home, void *data)
 {
-  if (home && data) {
+  if (!data)
+    return;
+
+  if (home) {
     su_alloc_t *allocation;
     su_block_t *sub = MEMLOCK(home);
 
@@ -822,7 +835,28 @@ void su_free(su_home_t *home, void *data)
     UNLOCK(home);
   }
 
-  free(data);
+  safefree(data);
+}
+
+/** Check if pointer has been allocated through home.
+ *
+ * @param home   pointer to a memory home
+ * @param data   pointer to a memory area possibly allocated though home
+ */
+int su_home_check_alloc(su_home_t const *home, void const *data)
+{
+  int retval = 0;
+
+  if (home && data) {
+    su_block_t const *sub = MEMLOCK(home);
+    su_alloc_t *allocation = su_block_find(sub, data);
+
+    retval = allocation != NULL;
+
+    UNLOCK(home);
+  }
+
+  return retval;
 }
 
 /** Check home consistency.
@@ -915,6 +949,11 @@ void su_home_destroy(su_home_t *home)
  * @retval -1 upon an error.
  *
  * @sa SU_HOME_INIT(), su_home_deinit(), su_home_new(), su_home_clone()
+ *
+ * @bug 
+ * Prior to @VERSION_1_12_8 the su_home_t structure should have been
+ * initialized with SU_HOME_INIT() or otherwise zeroed before calling
+ * su_home_init().
  */
 int su_home_init(su_home_t *home)
 {
@@ -924,6 +963,8 @@ int su_home_init(su_home_t *home)
     return -1;
 
   home->suh_blocks = sub = su_hash_alloc(SUB_N);
+  home->suh_lock = NULL;
+
   if (!sub)
     return -1;
 
@@ -965,7 +1006,7 @@ void _su_home_deinit(su_home_t *home)
 	}
 	else if (su_is_preloaded(b, b->sub_nodes[i].sua_data))
 	  continue;
-	free(b->sub_nodes[i].sua_data);
+	safefree(b->sub_nodes[i].sua_data);
       }
     }
 
@@ -979,7 +1020,7 @@ void _su_home_deinit(su_home_t *home)
     home->suh_blocks = NULL;
 
     if (home->suh_lock)
-      su_home_destroy_mutexes(home->suh_lock);
+      _su_home_destroy_mutexes(home->suh_lock);
   }
 
   home->suh_lock = NULL;
@@ -1046,7 +1087,7 @@ int su_home_move(su_home_t *dst, su_home_t *src)
       else
 	used = s->sub_used;
 
-      if ((used && d == NULL) || 3 * used > 2 * d->sub_n) {
+      if (used && (d == NULL || 3 * used > 2 * d->sub_n)) {
 	if (d)
 	  for (n = n2 = d->sub_n; 3 * used > 2 * n2; n2 = 4 * n2 + 3)
 	    ;
@@ -1080,7 +1121,9 @@ int su_home_move(su_home_t *dst, su_home_t *src)
 	d = d2;
       }
 
-      if ((n = s->sub_n)) {
+      if (s->sub_used) {
+	n = s->sub_n;
+
 	for (i = 0; i < n; i++)
 	  if (s->sub_nodes[i].sua_data) {
 	    su_block_add(d, s->sub_nodes[i].sua_data)[0] = s->sub_nodes[i];
@@ -1092,6 +1135,7 @@ int su_home_move(su_home_t *dst, su_home_t *src)
       }
 
       if (s->sub_stats) {
+				/* XXX */
       }
     }
 
@@ -1454,42 +1498,126 @@ int su_home_is_threadsafe(su_home_t const *home)
   return home && home->suh_lock;
 }
 
-/** Obtain exclusive lock on home (if home is threadsafe). */
+/** Increase refcount and obtain exclusive lock on home. 
+ *
+ * @note The #su_home_t structure must be created with su_home_new() or
+ * su_home_clone(), or initialized with su_home_init() before using this
+ * function. 
+ *
+ * In order to enable actual locking, use su_home_threadsafe(), too. 
+ * Otherwise the su_home_mutex_lock() will just increase the reference
+ * count.
+ */
 int su_home_mutex_lock(su_home_t *home)
 {
+  int error;
+
   if (home == NULL)
     return su_seterrno(EFAULT);
 
-  if (home->suh_lock) {
-    su_home_ref(home);
-    su_home_mutex_locker(home->suh_lock);
-  }
-  else if (home->suh_blocks) {
-    if (!su_home_ref(home))
-      return -1;
-  }
+  if (home->suh_blocks == NULL || !su_home_ref(home))
+    return su_seterrno(EINVAL);  /* Uninitialized home */
+
+  if (!home->suh_lock)
+    return 0;			/* No-op */
+
+  error = _su_home_mutex_locker(home->suh_lock);
+  if (error)
+    return su_seterrno(error);
 
   return 0;
 }
 
-/** Release exclusive lock on home (if home is threadsafe) */
+/** Release exclusive lock on home and decrease refcount (if home is threadsafe).
+ *
+ * @sa su_home_unlock().
+ */
 int su_home_mutex_unlock(su_home_t *home)
 {
   if (home == NULL)
     return su_seterrno(EFAULT);
 
   if (home->suh_lock) {
-    su_home_mutex_unlocker(home->suh_lock);
-    su_home_unref(home);
-  }
-  else if (home->suh_blocks) {
-    su_home_unref(home);
+    int error = _su_home_mutex_unlocker(home->suh_lock);
+    if (error)
+      return su_seterrno(error);
   }
 
+  if (home->suh_blocks == NULL)
+    return su_seterrno(EINVAL), -1; /* Uninitialized home */
+
+  su_home_unref(home);
+
   return 0;
 }
 
 
+/** Obtain exclusive lock on home without increasing refcount. 
+ *
+ * Unless su_home_threadsafe() has been used to intialize locking on home
+ * object the function just returns -1.
+ *
+ * @return 0 if successful, -1 if not threadsafe, error code otherwise.
+ *
+ * @sa su_home_mutex_lock(), su_home_unlock(), su_home_trylock().
+ *
+ * @NEW_1_12_8
+ */
+int su_home_lock(su_home_t *home)
+{
+  if (home == NULL)
+    return EFAULT;
+
+  if (home->suh_lock == NULL)
+    return -1;			/* No-op */
+
+  return _su_home_mutex_locker(home->suh_lock);
+}
+
+
+/** Try to obtain exclusive lock on home without increasing refcount. 
+ *
+ * @return 0 if successful, -1 if not threadsafe, 
+ * EBUSY if already locked, error code otherwise.
+ *
+ * @sa su_home_lock(), su_home_unlock().
+ *
+ * @NEW_1_12_8
+ */
+int su_home_trylock(su_home_t *home)
+{
+  if (home == NULL)
+    return EFAULT;
+
+  if (home->suh_lock == NULL)
+    return -1;			/* No-op */
+
+  return _su_home_mutex_trylocker(home->suh_lock);
+}
+
+
+/** Release exclusive lock on home.
+ *
+ * Release lock without decreasing refcount.
+ *
+ * @return 0 if successful, -1 if not threadsafe, error code otherwise.
+ *
+ * @sa su_home_lock(), su_home_trylock(), su_home_mutex_unlock().
+ *
+ * @NEW_1_12_8
+ */
+int su_home_unlock(su_home_t *home)
+{
+  if (home == NULL)
+    return EFAULT;
+
+  if (home->suh_lock == NULL)
+    return -1;			/* No-op */
+
+  return _su_home_mutex_unlocker(home->suh_lock);
+}
+
+
 /** Initialize statistics structure */
 void su_home_init_stats(su_home_t *home)
 {
@@ -1523,7 +1651,7 @@ void su_home_init_stats(su_home_t *home)
 /** Retrieve statistics from memory home.
  */
 void su_home_get_stats(su_home_t *home, int include_clones, 
-		       su_home_stat_t hs[1],
+		       su_home_stat_t *hs,
 		       isize_t size)
 {
   su_block_t *sub;
@@ -1531,7 +1659,7 @@ void su_home_get_stats(su_home_t *home, int include_clones,
   if (hs == NULL || size < (sizeof hs->hs_size))
     return;
 
-  memset(hs, 0, size);
+  memset((void *)hs, 0, size);
 
   sub = MEMLOCK(home);
 
