@@ -268,8 +268,14 @@ static int su_base_port_execute_msgs(su_msg_t *queue)
 
     queue = msg->sum_next, msg->sum_next = NULL;
 
-    if (f) 
-      f(SU_ROOT_MAGIC(msg->sum_to->sut_root), &msg, msg->sum_data);
+    if (f) {
+      su_root_t *root = msg->sum_to->sut_root;
+
+      if (msg->sum_to->sut_port == NULL)
+	msg->sum_to->sut_root = NULL;
+      f(SU_ROOT_MAGIC(root), &msg, msg->sum_data);
+    }
+
     su_msg_delivery_report(&msg);
     n++;
   }
@@ -554,11 +560,18 @@ static void su_base_port_clone_break(su_root_magic_t *m,
   su_root_destroy(task->sut_root);
 }
 
+/**Wait for the clone to exit.
+ * @internal
+ *
+ * Called by su_port_wait() and su_clone_wait()
+ */ 
 void su_base_port_wait(su_clone_r rclone)
 {
   su_port_t *self;
   su_root_t *root_to_wait;
 
+  assert(*rclone);
+
   self = su_msg_from(rclone)->sut_port;
   assert(self == su_msg_to(rclone)->sut_port);
   root_to_wait = su_msg_to(rclone)->sut_root;
