@@ -97,7 +97,7 @@ void su_pthread_port_lock(su_port_t *self, char const *who)
   PORT_LOCK_DEBUG(("%p at %s locking(%p)...",
 		   (void *)pthread_self(), who, self));
 
-  su_home_mutex_lock(self->sup_base->sup_home);
+  su_home_lock(self->sup_base->sup_home);
 
   PORT_LOCK_DEBUG((" ...%p at %s locked(%p)...", 
 		   (void *)pthread_self(), who, self));
@@ -105,7 +105,7 @@ void su_pthread_port_lock(su_port_t *self, char const *who)
 
 void su_pthread_port_unlock(su_port_t *self, char const *who)
 {
-  su_home_mutex_unlock(self->sup_base->sup_home);
+  su_home_unlock(self->sup_base->sup_home);
 
   PORT_LOCK_DEBUG((" ...%p at %s unlocked(%p)\n", 
 		   (void *)pthread_self(), who, self));
@@ -388,6 +388,8 @@ static void su_pthread_port_clone_break(su_root_magic_t *m,
 
 /** Wait for the pthread clone to exit.
  * @internal
+ *
+ * Called by su_port_wait() and su_clone_wait().
  */
 void su_pthread_port_wait(su_clone_r rclone)
 {
@@ -395,6 +397,8 @@ void su_pthread_port_wait(su_clone_r rclone)
   struct su_pthread_port_waiting_parent mom[1];
   pthread_t tid;
 
+  assert(*rclone);
+
   clone = su_msg_to(rclone)->sut_port;
   parent = su_msg_from(rclone)->sut_port;
 
