@@ -22,7 +22,7 @@
  *
  */
 
-/**
+/**@internal
  * @file torture_rbtree.c
  * @brief Test red-black tree
  *
@@ -73,9 +73,9 @@ char const *name = "torture_rbtree";
 #define INSERT(node) ((void)0)
 #define REMOVE(node) ((node)->left = (node)->right = (node)->parent = NULL)
 
-RBTREE_PROTOS(static inline, redblack, Node);
+RBTREE_PROTOS(su_inline, redblack, Node);
 
-RBTREE_BODIES(static inline, redblack, Node, LEFT, RIGHT, PARENT,
+RBTREE_BODIES(su_inline, redblack, Node, LEFT, RIGHT, PARENT,
 	      IS_RED, SET_RED, IS_BLACK, SET_BLACK, COPY_COLOR,
 	      CMP, INSERT, REMOVE);
 
@@ -332,7 +332,7 @@ int test_balance(void)
     nodes[i] = node = node_new(home, i); TEST_1(node);
     TEST(redblack_insert(&tree, node, &o), 0);
     TEST_P(o, NULL);
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(i + 1 + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(i + 1 + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -363,7 +363,7 @@ int test_balance(void)
     TEST_1(node->parent == NULL && 
 	   node->left == NULL && 
 	   node->right == NULL); 
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -373,7 +373,7 @@ int test_balance(void)
     o = (void *)-1;
     TEST(redblack_insert(&tree, nodes[i], &o), 0); 
     TEST_P(o, NULL);
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -384,7 +384,7 @@ int test_balance(void)
 
   for (i = 0; i < N; i++) {
     redblack_remove(&tree, nodes[i]);
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -397,7 +397,7 @@ int test_balance(void)
     TEST(redblack_insert(&tree, nodes[sn], &o), 0); 
     nodes[sn]->inserted = 1;
     TEST_P(o, NULL);
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(i + 1 + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(i + 1 + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -412,7 +412,7 @@ int test_balance(void)
     TEST(nodes[sn]->inserted, 1);
     redblack_remove(&tree, nodes[sn]);
     nodes[sn]->inserted = 0;
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -425,7 +425,7 @@ int test_balance(void)
     TEST(redblack_insert(&tree, nodes[sn], &o), 0); 
     nodes[sn]->inserted = 1;
     TEST_P(o, NULL);
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(i + 1 + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(i + 1 + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -440,7 +440,7 @@ int test_balance(void)
     TEST(nodes[sn]->inserted, 1);
     redblack_remove(&tree, nodes[sn]);
     nodes[sn]->inserted = 0;
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -453,7 +453,7 @@ int test_balance(void)
     TEST(redblack_insert(&tree, nodes[sn], &o), 0); 
     nodes[sn]->inserted = 1;
     TEST_P(o, NULL);
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(i + 1 + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(i + 1 + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -468,7 +468,7 @@ int test_balance(void)
     TEST(nodes[sn]->inserted, 1);
     redblack_remove(&tree, nodes[sn]);
     nodes[sn]->inserted = 0;
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -504,7 +504,7 @@ int test_balance(void)
     TEST(node->inserted, 1);
     redblack_remove(&tree, node);
     node->inserted = 0;
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -540,7 +540,7 @@ int test_balance(void)
     TEST(node->inserted, 1);
     redblack_remove(&tree, node);
     node->inserted = 0;
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -576,7 +576,7 @@ int test_balance(void)
     TEST(node->inserted, 1);
     redblack_remove(&tree, node);
     node->inserted = 0;
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -630,7 +630,7 @@ int test_balance(void)
     TEST(node->inserted, 1);
     redblack_remove(&tree, node);
     node->inserted = 0;
-    TEST_1(redblack_height(tree) <= 2 * log2ceil(N - i + 1));
+    TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(N - i + 1));
     TEST_1(redblack_check(tree));
   }
 
@@ -648,7 +648,7 @@ int test_speed(void)
   Node *tree = NULL, *o = NULL;
   Node *node;
   unsigned i;
-  int const N = 1000000;
+  unsigned int const N = 1000000U;
 
   BEGIN();
 
@@ -660,7 +660,7 @@ int test_speed(void)
     TEST_P(o, NULL);
   }
 
-  TEST_1(redblack_height(tree) <= 2 * log2ceil(i + 1));
+  TEST_1(redblack_height(tree) <= 2 * (int)log2ceil(i + 1));
 
   for (i = 0; i < N; i++) {
     node = node_find(tree, i);
@@ -681,11 +681,12 @@ int test_speed(void)
 }
 
 
-void usage(void)
+void usage(int exitcode)
 {
   fprintf(stderr,
-	  "usage: %s [-v]\n",
+	  "usage: %s [-v] [-a]\n",
 	  name);
+  exit(exitcode);
 }
 
 int main(int argc, char *argv[])
@@ -696,8 +697,10 @@ int main(int argc, char *argv[])
   for (i = 1; argv[i]; i++) {
     if (strcmp(argv[i], "-v") == 0)
       tstflags |= tst_verbatim;
+    else if (strcmp(argv[i], "-a") == 0)
+      tstflags |= tst_abort;
     else
-      usage();
+      usage(1);
   }
 
   retval |= test_insert(); fflush(stdout);
