@@ -155,7 +155,7 @@ static int test_sockaddr(void)
 int test_sendrecv(void)
 {
   su_socket_t s, l, a;
-  int n;
+  ssize_t n;
   su_sockaddr_t su, csu;
   socklen_t sulen = sizeof su.su_sin, csulen = sizeof csu.su_sin;
   char b1[8], b2[8], b3[8];
@@ -172,11 +172,12 @@ int test_sendrecv(void)
   BEGIN();
 
   s = su_socket(AF_INET, SOCK_DGRAM, 0); TEST_1(s != -1);
+  su_setblocking(s, 1);
 
   memset(&su, 0, sulen);
   su.su_len = sulen;
   su.su_family = AF_INET;
-  TEST(inet_pton(AF_INET, "127.0.0.1", &su.su_sin.sin_addr), 1);
+  TEST(su_inet_pton(AF_INET, "127.0.0.1", &su.su_sin.sin_addr), 1);
 
   TEST(bind(s, &su.su_sa, sulen), 0);
   TEST(getsockname(s, &su.su_sa, &sulen), 0);
@@ -196,7 +197,7 @@ int test_sendrecv(void)
   memset(&su, 0, sulen);
   su.su_len = sulen;
   su.su_family = AF_INET;
-  TEST(inet_pton(AF_INET, "127.0.0.1", &su.su_sin.sin_addr), 1);
+  TEST(su_inet_pton(AF_INET, "127.0.0.1", &su.su_sin.sin_addr), 1);
 
   TEST(bind(l, &su.su_sa, sulen), 0);
   TEST(bind(s, &su.su_sa, sulen), 0);
@@ -209,6 +210,8 @@ int test_sendrecv(void)
   TEST(connect(s, &su.su_sa, sulen), 0);
   a = accept(l, &csu.su_sa, &csulen); TEST_1(a != -1);
 
+  TEST(su_setblocking(a, 1), 0);
+
   n = su_vsend(s, sv, 3, 0, NULL, 0); TEST(n, 8 + 8 + 6);
   n = su_vrecv(a, rv, 3, 0, NULL, NULL); TEST(n, 8 + 8 + 6);
 
@@ -301,7 +304,7 @@ int test_select(void)
   memset(&su, 0, sulen);
   su.su_len = sulen;
   su.su_family = AF_INET;
-  TEST(inet_pton(AF_INET, "127.0.0.1", &su.su_sin.sin_addr), 1);
+  TEST(su_inet_pton(AF_INET, "127.0.0.1", &su.su_sin.sin_addr), 1);
   TEST(bind(s, &su.su_sa, sulen), 0);
   TEST(getsockname(s, &su.su_sa, &sulen), 0);
   
