@@ -26,8 +26,9 @@
 /** Defined when <tport_internal.h> has been included. */
 #define TPORT_INTERNAL_H
 
-/**@file tport_internal.h
- * @brief Transport interface
+/**@internal
+ * @file tport_internal.h
+ * @brief Internal implementation of transport interface
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>
  *
@@ -61,11 +62,17 @@
 
 #include <sofia-sip/su_debug.h>
 
-#if !defined(MSG_NOSIGNAL) || defined(__CYGWIN__)
+#if !defined(MSG_NOSIGNAL) || defined(__CYGWIN__) || defined(SYMBIAN)
 #undef MSG_NOSIGNAL
 #define MSG_NOSIGNAL (0)
 #endif
 
+#if (_WIN32_WINNT >= 0x0600)
+#ifndef HAVE_MSG_TRUNC
+#define HAVE_MSG_TRUNC 1
+#endif
+#endif
+
 #if !HAVE_MSG_TRUNC
 #define MSG_TRUNC (0)
 #endif
@@ -89,11 +96,14 @@ struct sigcomp_compartment;
 
 typedef long unsigned LU; 	/* for printf() and friends */
 
-/** Transport parameters */
+/** @internal Transport parameters */
 typedef struct {
   unsigned tpp_mtu;		/**< Maximum packet size */
   unsigned tpp_idle;		/**< Allowed connection idle time. */
   unsigned tpp_timeout;		/**< Allowed idle time for message. */
+  unsigned tpp_keepalive;	/**< Keepalive PING interval */
+  unsigned tpp_pingpong;	/**< PONG-to-PING interval */
+
   unsigned tpp_sigcomp_lifetime;  /**< SigComp compartment lifetime  */
   unsigned tpp_thrpsize;	/**< Size of thread pool */
 
@@ -106,13 +116,14 @@ typedef struct {
   unsigned tpp_conn_orient:1;   /**< Connection-orienteded */
   unsigned tpp_sdwn_error:1;	/**< If true, shutdown is error. */
   unsigned tpp_stun_server:1;	/**< If true, use stun server */
+  unsigned tpp_pong2ping:1;	/**< If true, respond with pong to ping */
 
   unsigned :0;
 
 } tport_params_t;
 
 
-/** Transport object.
+/** @internal Transport object.
  *
  * A transport object can be used in three roles, to represent transport
  * list (aka master transport), to represent available transports (aka
@@ -122,7 +133,7 @@ typedef struct {
 struct tport_s {
   su_home_t           tp_home[1];       /**< Memory home */
 
-  int                 tp_refs;		/**< Number of references to tport */
+  ssize_t             tp_refs;		/**< Number of references to tport */
 
   unsigned            tp_black:1;       /**< Used by red-black-tree */
   
@@ -130,8 +141,13 @@ struct tport_s {
   unsigned            tp_conn_orient:1;	/**< Is connection-oriented */
   unsigned            tp_has_connection:1; /**< Has real connection */
   unsigned            tp_reusable:1;    /**< Can this connection be reused */
-  unsigned            tp_closed : 1;    /**< This transport is closed */
-  /**< Remote end has sent FIN (2) or we should not just read */
+  unsigned            tp_closed : 1;
+  /**< This transport is closed.
+   * 
+   * A closed transport is inserted into pri_closed list.
+   */
+
+  /** Remote end has sent FIN (2) or we should not just read */
   unsigned            tp_recv_close:2;
   /** We will send FIN (1) or have sent FIN (2) */
   unsigned            tp_send_close:2; 
@@ -150,10 +166,10 @@ struct tport_s {
 
   tp_magic_t         *tp_magic; 	/**< Context provided by consumer */
 
-  msg_t const        *tp_rlogged;       /**< Last logged when receiving */
-  msg_t const        *tp_slogged;       /**< Last logged when sending */
+  su_timer_t         *tp_timer;	        /**< Timer object */
 
-  unsigned            tp_time;	        /**< When this transport was last used */
+  su_time_t           tp_ktime;	        /**< Keepalive timer updated */
+  su_time_t           tp_ptime;	        /**< Ping sent */
 
   tp_name_t           tp_name[1];	/**< Transport name.
 					 * 
@@ -178,16 +194,18 @@ struct tport_s {
   /* ==== Receive queue ================================================== */
 
   msg_t   	     *tp_msg;		/**< Message being received */
+  msg_t const        *tp_rlogged;       /**< Last logged when receiving */
+  su_time_t           tp_rtime;	        /**< Last time received data */
+  unsigned short      tp_ping;	        /**< Whitespace ping being received */
 
   /* ==== Pending messages =============================================== */
 
-  tport_pending_t    *tp_pending;       /**< Pending requests */
-  tport_pending_t    *tp_released;      /**< Released pends */
+  unsigned short      tp_reported;      /**< Report counter */
   unsigned            tp_plen;          /**< Size of tp_pending */
   unsigned            tp_pused;         /**< Used pends */
-  unsigned short      tp_reported;      /**< Report counter */
-  unsigned short      tp_pad;
-
+  tport_pending_t    *tp_pending;       /**< Pending requests */
+  tport_pending_t    *tp_released;      /**< Released pends */
+  
   /* ==== Send queue ===================================================== */
 
   msg_t             **tp_queue;		/**< Messages being sent */
@@ -199,6 +217,9 @@ struct tport_s {
   msg_iovec_t        *tp_iov;		/**< Iovecs allocated for sending */
   size_t              tp_iovlen;	/**< Number of allocated iovecs */
 
+  msg_t const        *tp_slogged;       /**< Last logged when sending */
+  su_time_t           tp_stime;	        /**< Last time sent message */
+
   /* ==== Extensions  ===================================================== */
 
   tport_compressor_t *tp_comp;
@@ -206,12 +227,12 @@ struct tport_s {
   /* ==== Statistics  ===================================================== */
   
   struct {
-    uint64_t sent_bytes, sent_on_line, recv_bytes, recv_on_line;
-    uint64_t sent_msgs, recv_msgs;
+    uint64_t sent_msgs, sent_errors, sent_bytes, sent_on_line;
+    uint64_t recv_msgs, recv_errors, recv_bytes, recv_on_line;
   } tp_stats;
 };
 
-/** Primary structure */
+/** @internal Primary structure */
 struct tport_primary {
   tport_t             pri_primary[1];   /**< Transport part */
 #if DOXYGEN_ONLY
@@ -228,10 +249,10 @@ struct tport_primary {
 					 * tport_type_stun, etc. 
 					 */
 
-  char                pri_ident[16];
   tport_primary_t    *pri_next;	        /**< Next primary tport */
 
-  tport_t            *pri_secondary;	/**< Secondary tports */
+  tport_t            *pri_open;		/**< Open secondary tports */
+  tport_t            *pri_closed;       /**< Closed secondary tports */
 
   unsigned            pri_updating:1;   /**< Currently updating address */
   unsigned            pri_natted:1;	/**< Using natted address  */
@@ -241,10 +262,9 @@ struct tport_primary {
   void               *pri_stun_handle;
 
   tport_params_t      pri_params[1];      /**< Transport parameters */
-
 };
 
-/** Master structure */
+/** @internal Master structure */
 struct tport_master {
   tport_t             mr_master[1];
 #if DOXYGEN_ONLY
@@ -263,9 +283,9 @@ struct tport_master {
 
   /**< Timer reclaiming unused connections and compartment */
   su_timer_t         *mr_timer;		
-  /** File to dump received and sent data */
+  /** FILE to dump received and sent data */
   FILE               *mr_dump_file;	
-
+  char               *mr_dump;	/**< Filename for dumping received/sent data */
   tport_primary_t    *mr_primaries;        /**< List of primary contacts */
 
   tport_params_t      mr_params[1];
@@ -307,7 +327,7 @@ struct tport_master {
 #endif
 };
 
-/** Virtual funtion table for transports */
+/** @internal Virtual function table for transports */
 struct tport_vtable
 {
   char const *vtp_name;
@@ -344,17 +364,20 @@ struct tport_vtable
   int (*vtp_stun_response)(tport_t const *self,
 			   void *msg, size_t msglen,
 			   void *addr, socklen_t addrlen);
+  int (*vtp_next_secondary_timer)(tport_t *self, su_time_t *, 
+				  char const **return_why);
+  void (*vtp_secondary_timer)(tport_t *self, su_time_t);
 };
 
 int tport_register_type(tport_vtable_t const *vtp);
 
-/** Test if transport is needs connect() before sending. */
+/** @internal Test if transport needs connect() before sending. */
 su_inline int tport_is_connection_oriented(tport_t const *self)
 {
   return self->tp_conn_orient;
 }
 
-/** Test if transport involves connection. @NEW_1_12_5 */
+/** @internal Test if transport involves connection. @NEW_1_12_5. */
 su_inline int tport_has_connection(tport_t const *self)
 {
   return self->tp_has_connection;
@@ -388,11 +411,16 @@ tport_t *tport_alloc_secondary(tport_primary_t *pri,
 int tport_accept(tport_primary_t *pri, int events);
 void tport_zap_secondary(tport_t *self);
 
+int tport_set_secondary_timer(tport_t *self);
+void tport_base_timer(tport_t *self, su_time_t now);
+
 int tport_bind_socket(int socket,
 		      su_addrinfo_t *ai,
 		      char const **return_culprit);
 void tport_close(tport_t *self);
+int tport_shutdown0(tport_t *self, int how);
 
+int tport_has_queued(tport_t const *self);
 
 int tport_error_event(tport_t *self);
 void tport_recv_event(tport_t *self);
@@ -414,6 +442,8 @@ int tport_send_msg(tport_t *self, msg_t *msg,
 		   tp_name_t const *tpn, 
 		   struct sigcomp_compartment *cc);
 
+void tport_send_queue(tport_t *self);
+
 void tport_deliver(tport_t *self, msg_t *msg, msg_t *next, 
 		   tport_compressor_t *comp,
 		   su_time_t now);
@@ -423,13 +453,16 @@ int tport_recv_error_report(tport_t *self);
 void tport_error_report(tport_t *self, int errcode, 
 			su_sockaddr_t const *addr);
 
-void tport_open_log(tport_master_t *mr, tagi_t *tags);
+int tport_open_log(tport_master_t *mr, tagi_t *tags);
 void tport_log_msg(tport_t *tp, msg_t *msg, char const *what, 
 		   char const *via, su_time_t now);
 void tport_dump_iovec(tport_t const *self, msg_t *msg, 
 		      size_t n, su_iovec_t const iov[], size_t iovused,
 		      char const *what, char const *how);
 
+int tport_tcp_ping(tport_t *self, su_time_t now);
+int tport_tcp_pong(tport_t *self);
+
 extern tport_vtable_t const tport_udp_vtable;
 extern tport_vtable_t const tport_udp_client_vtable;
 
@@ -461,6 +494,15 @@ int tport_recv_stream(tport_t *self);
 ssize_t tport_send_stream(tport_t const *self, msg_t *msg,
 			  msg_iovec_t iov[], size_t iovused);
 
+int tport_tcp_next_timer(tport_t *self, su_time_t *, char const **);
+void tport_tcp_timer(tport_t *self, su_time_t);
+
+int tport_next_recv_timeout(tport_t *, su_time_t *, char const **);
+void tport_recv_timeout_timer(tport_t *self, su_time_t now);
+
+int tport_next_keepalive(tport_t *self, su_time_t *, char const **);
+void tport_keepalive_timer(tport_t *self, su_time_t now);
+
 extern tport_vtable_t const tport_sctp_vtable;
 extern tport_vtable_t const tport_sctp_client_vtable;
 extern tport_vtable_t const tport_tls_vtable;
@@ -492,6 +534,12 @@ int tport_recv_stun_dgram(tport_t const *self, msg_t **in_out_msg,
 int tport_stun_server_add_socket(tport_t *tp);
 int tport_stun_server_remove_socket(tport_t *tp);
 
+void tport_recv_bytes(tport_t *self, ssize_t bytes, ssize_t on_line);
+void tport_recv_message(tport_t *self, msg_t *msg, int error);
+
+void tport_sent_bytes(tport_t *self, ssize_t bytes, ssize_t on_line);
+void tport_sent_message(tport_t *self, msg_t *msg, int error);
+
 /* ---------------------------------------------------------------------- */
 /* Compressor plugin */
 extern tport_comp_vtable_t const *tport_comp_vtable;
