@@ -52,13 +52,13 @@
 
 typedef struct tport_sigcomp_handler tport_sigcomp_handler_t;
 
-/** Per end-point SigComp data */
+/** @internal Per end-point SigComp data */
 struct tport_compressor {
   struct sigcomp_state_handler *msc_state_handler;
   struct sigcomp_compartment *msc_compartment;
 };
 
-/** Per-socket SigComp data */
+/** @internal Per-socket SigComp data */
 struct tport_comp {
   struct sigcomp_udvm          *sc_udvm;
   struct sigcomp_compartment   *sc_cc;
@@ -811,14 +811,14 @@ tport_sigcomp_deliver(tport_t *self, msg_t *msg, su_time_t now)
 
 #if HAVE_SIGCOMP && 0
 
-static inline
+su_inline
 int msg_is_compressed(msg_t *msg)
 {
   return msg && 
     (msg_addrinfo(msg)->ai_flags & TP_AI_COMPRESSED) == TP_AI_COMPRESSED;
 }
 
-static inline
+su_inline
 void msg_mark_as_compressed(msg_t *msg)
 {
   if (msg)
