@@ -61,6 +61,7 @@
 #endif
 
 #include "tport_tls.h"
+#include "tport_internal.h"
 
 char const tls_version[] = OPENSSL_VERSION_TEXT;
 
@@ -70,7 +71,6 @@ struct tls_s {
   SSL_CTX *ctx;
   SSL *con;
   BIO *bio_con;
-  BIO *bio_err;
   int type;
   int verified;
 
@@ -90,6 +90,35 @@ struct tls_s {
 
 enum { tls_buffer_size = 16384 };
 
+/** Log TLS error(s).
+ *
+ * Log the TLS error specified by the error code @a e and all the errors in
+ * the queue. The error code @a e implies no error, and it is not logged.
+ */
+static
+void tls_log_errors(unsigned level, char const *s, unsigned long e)
+{
+  if (e == 0)
+    e = ERR_get_error();
+
+  if (!tport_log->log_init)
+    su_log_init(tport_log);
+
+  if (s == NULL) s = "tls";
+
+  for (; e != 0; e = ERR_get_error()) {
+    if (level <= tport_log->log_level) {
+      const char *error = ERR_lib_error_string(e);
+      const char *func = ERR_func_error_string(e);
+      const char *reason = ERR_reason_error_string(e);
+
+      su_llog(tport_log, level, "%s: %08lx:%s:%s:%s\n", 
+	      s, e, error, func, reason);
+    }
+  }
+}
+
+
 static
 tls_t *tls_create(int type)
 {
@@ -101,6 +130,7 @@ tls_t *tls_create(int type)
   return tls;
 }
 
+
 static
 void tls_set_default(tls_issues_t *i)
 {
@@ -116,28 +146,24 @@ void tls_set_default(tls_issues_t *i)
   /* TLS_RSA_WITH_3DES_EDE_CBC_SHA; */
 }
 
+
 static
 int tls_verify_cb(int ok, X509_STORE_CTX *store)
 {
-  char data[256];
-
-  X509 *cert = X509_STORE_CTX_get_current_cert(store);
-  int  depth = X509_STORE_CTX_get_error_depth(store);
-  int  err = X509_STORE_CTX_get_error(store);
-
-#if nomore 
-  509_NAME_oneline(X509_get_subject_name(cert), data, 256);
-  fprintf(stderr,"depth=%d %s\n",depth,data);
-#endif
-
   if (!ok)
   {
-    fprintf(stderr, "-Error with certificate at depth: %i\n", depth);
+    char data[256];
+
+    X509 *cert = X509_STORE_CTX_get_current_cert(store);
+    int  depth = X509_STORE_CTX_get_error_depth(store);
+    int  err = X509_STORE_CTX_get_error(store);
+
+    SU_DEBUG_1(("-Error with certificate at depth: %i\n", depth));
     X509_NAME_oneline(X509_get_issuer_name(cert), data, 256);
-    fprintf(stderr, "  issuer   = %s\n", data);
+    SU_DEBUG_1(("  issuer   = %s\n", data));
     X509_NAME_oneline(X509_get_subject_name(cert), data, 256);
-    fprintf(stderr, "  subject  = %s\n", data);
-    fprintf(stderr, "  err %i:%s\n", err, X509_verify_cert_error_string(err));
+    SU_DEBUG_1(("  subject  = %s\n", data));
+    SU_DEBUG_1(("  err %i:%s\n", err, X509_verify_cert_error_string(err)));
   }
  
   return 1;			/* Always return "ok" */
@@ -156,9 +182,9 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
     if (ti->randFile &&
 	!RAND_load_file(ti->randFile, 1024 * 1024)) {
       if (ti->configured > 1) {
-	BIO_printf(tls->bio_err, "%s: cannot open randFile %s\n", 
-		   "tls_init_context", ti->randFile);
-	ERR_print_errors(tls->bio_err);
+	SU_DEBUG_3(("%s: cannot open randFile %s\n", 
+		   "tls_init_context", ti->randFile));
+	tls_log_errors(3, "tls_init_context", 0);
       }
       /* errno = EIO; */
       /* return -1; */
@@ -170,9 +196,6 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
   signal(SIGPIPE, SIG_IGN);
 #endif
 
-  if (tls->bio_err == NULL)
-    tls->bio_err = BIO_new_fp(stderr, BIO_NOCLOSE);
-
   if (tls->ctx == NULL) {
     SSL_METHOD *meth;
 
@@ -188,7 +211,7 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
   }
 
   if (tls->ctx == NULL) {
-    ERR_print_errors(tls->bio_err);
+    tls_log_errors(1, "tls_init_context", 0);
     errno = EIO;
     return -1;
   }
@@ -197,9 +220,9 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
 				    ti->cert,
 				    SSL_FILETYPE_PEM)) {
     if (ti->configured > 0) {
-      BIO_printf(tls->bio_err, "%s: invalid certificate: %s\n",
-		 "tls_init_context", ti->cert);
-      ERR_print_errors(tls->bio_err);
+      SU_DEBUG_1(("%s: invalid local certificate: %s\n",
+		 "tls_init_context", ti->cert));
+      tls_log_errors(1, "tls_init_context", 0);
 #if require_client_certificate
       errno = EIO;
       return -1;
@@ -211,7 +234,7 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
                                    ti->key, 
                                    SSL_FILETYPE_PEM)) {
     if (ti->configured > 0) {
-      ERR_print_errors(tls->bio_err);
+      tls_log_errors(1, "tls_init_context", 0);
 #if require_client_certificate
       errno = EIO;
       return -1;
@@ -221,8 +244,8 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
 
   if (!SSL_CTX_check_private_key(tls->ctx)) {
     if (ti->configured > 0) {
-      BIO_printf(tls->bio_err,
-		 "Private key does not match the certificate public key\n");
+      SU_DEBUG_1(("%s: private key does not match the certificate public key\n",
+		  "tls_init_context"));
     }
 #if require_client_certificate
     errno = EIO;
@@ -234,7 +257,7 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
                                      ti->CAfile, 
                                      ti->CApath)) {
     if (ti->configured > 0)
-      ERR_print_errors(tls->bio_err);
+      tls_log_errors(1, "tls_init_context", 0);
     errno = EIO;
     return -1;
   }
@@ -247,8 +270,8 @@ int tls_init_context(tls_t *tls, tls_issues_t const *ti)
                      tls_verify_cb);
 
   if (!SSL_CTX_set_cipher_list(tls->ctx, ti->cipher)) {
-    BIO_printf(tls->bio_err,"error setting cipher list\n");
-    ERR_print_errors(tls->bio_err);
+    SU_DEBUG_1(("%s: error setting cipher list\n", "tls_init_context"));
+    tls_log_errors(1, "tls_init_context", 0);
     errno = EIO;
     return -1;
   }
@@ -275,11 +298,10 @@ void tls_free(tls_t *tls)
   if (tls->bio_con != NULL)
     BIO_free(tls->bio_con);
 
-  if (tls->bio_err != NULL && tls->type != tls_slave)
-    BIO_free(tls->bio_err);
-
   for (k = 0; k < TLS_MAX_HOSTS; k++)
-    free(tls->hosts[k]), tls->hosts[k] = NULL;
+    if (tls->hosts[k]) {
+      free(tls->hosts[k]), tls->hosts[k] = NULL;
+    }
 
   free(tls);
 }
@@ -331,8 +353,7 @@ tls_t *tls_init_master(tls_issues_t *ti)
     tls->bio_con = BIO_new_socket(sock, BIO_NOCLOSE);
 
     if (tls->bio_con == NULL) {
-      BIO_printf(tls->bio_err, "tls_init_master: BIO_new_socket failed\n");
-      ERR_print_errors(tls->bio_err);
+      tls_log_errors(1, "tls_init_master", 0);
       tls_free(tls);
       errno = EIO;
       return NULL;
@@ -343,63 +364,12 @@ tls_t *tls_init_master(tls_issues_t *ti)
   return tls;
 }
 
-#if 0
-#include <poll.h>
-
-static 
-int tls_accept(tls_t *tls)
-{
-  int ret = SSL_accept(tls->con);
-  int verify_result;
-
-  if (ret <= 0) {
-    int err = SSL_get_error(tls->con, ret);
-    switch(err) {
-    case SSL_ERROR_WANT_READ:
-      return errno = EAGAIN, tls->read_events = SU_WAIT_IN, 0;
-    case SSL_ERROR_WANT_WRITE:
-      return errno = EAGAIN, tls->read_events = SU_WAIT_OUT, 0;
-
-    default:    
-      BIO_printf(tls->bio_err, "SSL_connect failed: %d %s\n", 
-                 err,
-                 ERR_error_string(err, NULL));
-      ERR_print_errors(tls->bio_err);
-      return -1;
-    }
-  }
-
-  verify_result = SSL_get_verify_result(tls->con);
-
-  if (verify_result != X509_V_OK) {
-    BIO_printf(tls->bio_err, 
-               "Client certificate doesn't verify: %s\n",
-               X509_verify_cert_error_string(verify_result));
-#if 0
-    tls_free(tls);
-    return NULL;
-#endif
-  }
-
-  if (SSL_get_peer_certificate(tls->con) == NULL) {
-    BIO_printf(tls->bio_err, "Client didn't send certificate\n");
-#if 0
-    tls_free(tls);
-    return NULL;
-#endif
-  }
-
-  return 1;
-}
-#endif
-
 tls_t *tls_clone(tls_t *master, int sock, int accept)
 {
   tls_t *tls = tls_create(tls_slave);
 
   if (tls) {
     tls->ctx = master->ctx;
-    tls->bio_err = master->bio_err;
 
     if (!(tls->read_buffer = malloc(tls_buffer_size)))
       free(tls), tls = NULL;
@@ -413,8 +383,7 @@ tls_t *tls_clone(tls_t *master, int sock, int accept)
   tls->con = SSL_new(tls->ctx);
 
   if (tls->con == NULL) {
-    BIO_printf(tls->bio_err, "tls_clone: SSL_new failed\n");
-    ERR_print_errors(tls->bio_err);
+    tls_log_errors(1, "tls_clone", 0);
     tls_free(tls);
     errno = EIO;
     return NULL;
@@ -567,10 +536,9 @@ int tls_check_hosts(tls_t *tls, char const *hosts[TLS_MAX_HOSTS])
 }
 
 static
-int tls_error(tls_t *tls, int ret, char const *who, char const *operation,
+int tls_error(tls_t *tls, int ret, char const *who,
 	      void *buf, int size)
 {
-  char errorbuf[128];
   int events = 0;
   int err = SSL_get_error(tls->con, ret);
 
@@ -594,9 +562,7 @@ int tls_error(tls_t *tls, int ret, char const *who, char const *operation,
     return -1;
 
   default:
-    BIO_printf(tls->bio_err, "%s: %s failed (%d): %s\n", 
-	       who, operation, err, ERR_error_string(err, errorbuf));
-    ERR_print_errors(tls->bio_err);
+    tls_log_errors(1, who, err);
     errno = EIO;
     return -1;
   }
@@ -623,9 +589,9 @@ ssize_t tls_read(tls_t *tls)
   }
 
   if (0)
-    fprintf(stderr, "tls_read(%p) called on %s (events %u)\n", (void *)tls,
+    SU_DEBUG_1(("tls_read(%p) called on %s (events %u)\n", (void *)tls,
 	    tls->type == tls_slave ? "server" : "client",
-	    tls->read_events);
+	    tls->read_events));
 
   if (tls->read_buffer_len)
     return (ssize_t)tls->read_buffer_len;
@@ -634,7 +600,7 @@ ssize_t tls_read(tls_t *tls)
 
   ret = SSL_read(tls->con, tls->read_buffer, tls_buffer_size);
   if (ret <= 0)
-    return tls_error(tls, ret, "tls_read", "SSL_read", NULL, 0);
+    return tls_error(tls, ret, "tls_read: SSL_read", NULL, 0);
 
   if (!tls->verified) {
     int err = tls_post_connection_check(tls);
@@ -643,9 +609,9 @@ ssize_t tls_read(tls_t *tls)
 	err != SSL_ERROR_SYSCALL &&
 	err != SSL_ERROR_WANT_WRITE &&
 	err != SSL_ERROR_WANT_READ) {
-      BIO_printf(tls->bio_err, 
+      SU_DEBUG_1((
 		 "%s: server certificate doesn't verify\n", 
-		 "tls_read");
+		 "tls_read"));
     }
   }
 
@@ -695,9 +661,9 @@ ssize_t tls_write(tls_t *tls, void *buf, size_t size)
   ssize_t ret;
 
   if (0) 
-    fprintf(stderr, "tls_write(%p, %p, "MOD_ZU") called on %s\n", 
+    SU_DEBUG_1(("tls_write(%p, %p, "MOD_ZU") called on %s\n", 
 	    (void *)tls, buf, size,
-	    tls && tls->type == tls_slave ? "server" : "client");
+	    tls && tls->type == tls_slave ? "server" : "client"));
 
   if (tls == NULL || buf == NULL) {
     errno = EINVAL;
@@ -731,14 +697,14 @@ ssize_t tls_write(tls_t *tls, void *buf, size_t size)
 
   if (!tls->verified) {
     if (tls_post_connection_check(tls) != X509_V_OK) {
-      BIO_printf(tls->bio_err, 
-		 "tls_read: server certificate doesn't verify\n");
+      SU_DEBUG_1((
+		 "tls_read: server certificate doesn't verify\n"));
     }
   }
 
   ret = SSL_write(tls->con, buf, size);
   if (ret < 0)
-    return tls_error(tls, ret, "tls_write", "SSL_write", buf, size);
+    return tls_error(tls, ret, "tls_write: SSL_write", buf, size);
 
   return ret;
 }
