@@ -22,7 +22,7 @@
  *
  */
 
-/**
+/**@internal
  * @file urlmap.c
  * @brief Mapping with hierarchical URLs.
  *
@@ -38,6 +38,7 @@
 #include <string.h>
 #include <assert.h>
 #include <errno.h>
+#include <stdlib.h>
 
 #include "urlmap.h"
 
@@ -246,7 +247,7 @@ void red_black_balance_delete(UrlMap **top, UrlMap *um)
 }
 
 /** Compare paths. */
-static inline
+su_inline
 int urlmap_pathcmp(url_t const *a, url_t const *b, int *return_hostmatch)
 {
   int retval;
