@@ -45,6 +45,9 @@
 #define SU_HAVE_BSDSOCK         0
 #define SU_HAVE_STDINT          (0)
 #define SU_HAVE_NT              0
+
+/* note: on Windows 2000 and older (WINVER<=500), IPv6-tech-preview 
+ * is needed for IPv4 support as well, so SU_HAVE_IN6 must be set */
 #define SU_HAVE_IN6             1
 
 #define SU_HAVE_PTHREADS        (1)
@@ -101,7 +104,12 @@
 #define snprintf    _snprintf
 #endif
 #ifndef vsnprintf
-#define vsnprintf   _vsnprintf
+#ifndef _MSC_VER 
+#define vsnprintf _vsnprintf
+#endif
+#if _MSC_VER < 1500
+#define vsnprintf _vsnprintf
+#endif
 #endif
 
 #define srandom(x)    srand((x))
